/**
 * Copyright (C) 2012-2016  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class TableConfig
{

    private boolean allHeaderColumnsEnabled = false;

    private boolean fixedTableHeightEnabled = false;

    private boolean allColumnsEnabled = false;

    private boolean allRowsEnabled = false;

    private boolean quickSearchEnabled = false;

    private boolean quickSearchImmediateEnabled = false;

    private boolean quickSearchVisibleAtStartupEnabled = false;

    private boolean quickSearchHideEnabled = false;

    private boolean searchEnabled = false;

    private boolean searchFromattingEnabled = false;

    private boolean searchImmediateEnabled = false;

    private boolean searchVisibleAtStartupEnabled = false;

    private boolean searchHideEnabled = false;

    private boolean pageableTableEnabled = false;

    private boolean frozenTableEnabled = false;

    private boolean frozenTableVisibleEnabled = false;

    private boolean hideFrozenTableEnabled = false;

    private boolean resizeFrozenTableEnabled = false;

    private boolean emptyFrozenTableEnabled = false;

    private boolean freezeUnfreezeColumnsEnabled = false;

    private boolean freezeUnfreezeColumnsToolEnabled = false;

    private boolean freezeAnyColumnsEnabled = false;

    private boolean unfreezeAnyColumnsEnabled = false;

    private boolean hideUnhideColumnsEnabled = false;

    private boolean hideUnhideColumnsToolEnabled = false;

    private boolean hideAnyColumnsEnabled = false;

    private boolean newDataWhenHideUnhideColumnEnabled = false;

    private boolean highlightActiveRowEnabled = false;

    private boolean orderedFrozenTableEnabled = false;

    private boolean orderedTableEnabled = false;

    private boolean filterInfoEnabled = false;

    private boolean filteredFrozenTableEnabled = false;

    private boolean filteredTableEnabled = false;

    private boolean contextMenuForFilteredFrozenTableEnabled = false;

    private boolean contextMenuForFilteredTableEnabled = false;

    private boolean expandedFrozenHeaderCellEnabled = false;

    private boolean expandedFrozenHeaderCellAlwaysVisibleEnabled = false;

    private boolean expandedFrozenHeaderCellForActiveColumnEnabled = false;

    private boolean expandedFrozenBodyCellEnabled = false;

    private boolean expandedFrozenBodyCellAlwaysVisibleEnabled = false;

    private boolean expandedHeaderCellEnabled = false;

    private boolean expandedHeaderCellAlwaysVisibleEnabled = false;

    private boolean expandedHeaderCellForActiveColumnEnabled = false;

    private boolean expandedBodyCellEnabled = false;

    private boolean expandedBodyCellAlwaysVisibleEnabled = false;

    private boolean wordWrapInHeaderCellsEnabled = false;

    private boolean wordWrapInBodyCellsEnabled = false;

    private boolean oddAndEvenColoringRowsEnabled = false;

    private boolean horizontalLinesEnabled = false;

    private boolean headerVerticalLinesEnabled = true;

    private boolean bodyVerticalLinesEnabled = true;

    private boolean loadingMessageEnabled = false;

    private boolean tooltipEnabled = false;

    private boolean wordWrapInTooltipEnabled = false;

    private boolean tableToolsMenuEnabled = false;

    private boolean tableToolsMenuAlwaysVisibleEnabled = false;

    private boolean tableToolsTrayEnabled = false;

    private int tableFrozenMinimalWidth = 0;

    private int tableMinimalWidth = 0;

    private int contextMenuDepthForFilteredFrozenTable = 0;

    private int contextMenuDepthForFilteredTable = 0;

    private int tableHeight = 0;

    private int pageSize = 0;

    private int pagerPages = 0;

    private int headerRowsAlignment = 2;

    private int headerRowHeight = 26;

    private int headerLineHeight = 22;

    private int headerCellMarginTop = 2;

    private int headerCellMarginRight = 4;

    private int headerCellMarginBottom = 2;

    private int headerCellMarginLeft = 4;

    private int bodyRowAlignments = 1;

    private int bodyRowHeight = 24;

    private int bodyLineHeight = 20;

    private int bodyCellMarginTop = 2;

    private int bodyCellMarginRight = 4;

    private int bodyCellMarginBottom = 2;

    private int bodyCellMarginLeft = 4;

    private int pageResizingActionDelay = 500;

    private int scrollbarActionDelay = 600;

    private int tableToolsMouseClickActionDelay = 100;

    private int tableToolsTrayMouseOverAnimationDuration = 300;

    private int tableToolsTrayAnimationDuration = 300;

    private int quickSearchAnimationDuration = 300;

    private int searchAnimationDuration = 300;

    private int loadingMessageShowDelay = 100;

    private int loadingMessageHideDelay = 300;

    private float loadingMessageOpacity = 0.5f;

    private String tableToolsPosition = "-top-right";

    private String filterInfoPosition = "bottom";

    private String tooltipPosition = null;

    private List<Integer> columnsFormatted = new ArrayList<>();

    private List<String> columnsFreezeEnabled = new ArrayList<>();

    private List<String> columnsUnfreezeEnabled = new ArrayList<>();

    private List<String> columnsHideEnabled = new ArrayList<>();

    private int tooltipMinimumWidth = 0;

    private int tooltipMaximumWidth = 0;

    public void processData( Map<String, String> tableConfigData )
    {
        if ( tableConfigData == null || tableConfigData.keySet()
                                                       .isEmpty() )
        {
            return;
        }

        if ( tableConfigData.containsKey( "frozenTableEnabled" ) )
        {
            frozenTableEnabled = Boolean.parseBoolean( tableConfigData.get( "frozenTableEnabled" ) );
        }

        if ( tableConfigData.containsKey( "frozenTableVisibleEnabled" ) )
        {
            frozenTableVisibleEnabled =
                frozenTableEnabled && Boolean.parseBoolean( tableConfigData.get( "frozenTableVisibleEnabled" ) );
        }

        if ( tableConfigData.containsKey( "hideFrozenTableEnabled" ) )
        {
            hideFrozenTableEnabled =
                frozenTableEnabled && Boolean.parseBoolean( tableConfigData.get( "hideFrozenTableEnabled" ) );
        }

        if ( tableConfigData.containsKey( "resizeFrozenTableEnabled" ) )
        {
            resizeFrozenTableEnabled =
                frozenTableEnabled && Boolean.parseBoolean( tableConfigData.get( "resizeFrozenTableEnabled" ) );
        }

        if ( tableConfigData.containsKey( "emptyFrozenTableEnabled" ) )
        {
            emptyFrozenTableEnabled =
                frozenTableEnabled && Boolean.parseBoolean( tableConfigData.get( "emptyFrozenTableEnabled" ) );
        }

        if ( tableConfigData.containsKey( "pageableTableEnabled" ) )
        {
            pageableTableEnabled = Boolean.parseBoolean( tableConfigData.get( "pageableTableEnabled" ) );
        }

        if ( tableConfigData.containsKey( "allHeaderColumnsEnabled" ) )
        {
            allHeaderColumnsEnabled = Boolean.parseBoolean( tableConfigData.get( "allHeaderColumnsEnabled" ) );
        }

        if ( tableConfigData.containsKey( "fixedTableHeightEnabled" ) )
        {
            fixedTableHeightEnabled = Boolean.parseBoolean( tableConfigData.get( "fixedTableHeightEnabled" ) );
        }

        if ( tableConfigData.containsKey( "allRowsEnabled" ) && !pageableTableEnabled )
        {
            allRowsEnabled = Boolean.parseBoolean( tableConfigData.get( "allRowsEnabled" ) );
        }

        if ( tableConfigData.containsKey( "allColumnsEnabled" ) )
        {
            allColumnsEnabled = Boolean.parseBoolean( tableConfigData.get( "allColumnsEnabled" ) );
        }

        if ( tableConfigData.containsKey( "columnNumberInWaitBoxEnabled" ) )
        {
            // table.showColumnNumbersInWaitBox(config
            // .getBoolean("columnNumberInWaitBoxEnabled"));
        }

        if ( tableConfigData.containsKey( "orderedFrozenTableEnabled" ) )
        {
            orderedFrozenTableEnabled =
                frozenTableEnabled && Boolean.parseBoolean( tableConfigData.get( "orderedFrozenTableEnabled" ) );
        }

        if ( tableConfigData.containsKey( "orderedTableEnabled" ) )
        {
            orderedTableEnabled = Boolean.parseBoolean( tableConfigData.get( "orderedTableEnabled" ) );
        }

        if ( tableConfigData.containsKey( "filteredFrozenTableEnabled" ) )
        {
            filteredFrozenTableEnabled =
                frozenTableEnabled && Boolean.parseBoolean( tableConfigData.get( "filteredFrozenTableEnabled" ) );
        }

        if ( tableConfigData.containsKey( "filteredTableEnabled" ) )
        {
            filteredTableEnabled = Boolean.parseBoolean( tableConfigData.get( "filteredTableEnabled" ) );
        }

        if ( tableConfigData.containsKey( "contextMenuForFilteredFrozenTableEnabled" ) )
        {
            contextMenuForFilteredFrozenTableEnabled = frozenTableEnabled && filteredFrozenTableEnabled
                && Boolean.parseBoolean( tableConfigData.get( "contextMenuForFilteredFrozenTableEnabled" ) );
        }

        if ( tableConfigData.containsKey( "contextMenuForFilteredTableEnabled" ) )
        {
            contextMenuForFilteredTableEnabled = filteredTableEnabled
                && Boolean.parseBoolean( tableConfigData.get( "contextMenuForFilteredTableEnabled" ) );
        }

        if ( tableConfigData.containsKey( "filterInfoEnabled" ) )
        {
            filterInfoEnabled = ( filteredFrozenTableEnabled || filteredTableEnabled )
                && Boolean.parseBoolean( tableConfigData.get( "filterInfoEnabled" ) );
        }

        if ( tableConfigData.containsKey( "quickSearchEnabled" ) )
        {
            quickSearchEnabled = ( filteredFrozenTableEnabled || filteredTableEnabled )
                && Boolean.parseBoolean( tableConfigData.get( "quickSearchEnabled" ) );
        }

        if ( tableConfigData.containsKey( "quickSearchImmediateEnabled" ) )
        {
            quickSearchImmediateEnabled =
                quickSearchEnabled && Boolean.parseBoolean( tableConfigData.get( "quickSearchImmediateEnabled" ) );
        }

        if ( tableConfigData.containsKey( "quickSearchVisibleAtStartupEnabled" ) )
        {
            quickSearchVisibleAtStartupEnabled = quickSearchEnabled
                && ( Boolean.parseBoolean( tableConfigData.get( "quickSearchVisibleAtStartupEnabled" ) )
                    || !Boolean.parseBoolean( tableConfigData.get( "quickSearchHideEnabled" ) ) );
        }

        if ( tableConfigData.containsKey( "quickSearchHideEnabled" ) )
        {
            quickSearchHideEnabled =
                quickSearchEnabled && Boolean.parseBoolean( tableConfigData.get( "quickSearchHideEnabled" ) );
        }

        if ( tableConfigData.containsKey( "searchEnabled" ) )
        {
            searchEnabled = ( filteredFrozenTableEnabled || filteredTableEnabled )
                && Boolean.parseBoolean( tableConfigData.get( "searchEnabled" ) );
        }

        if ( tableConfigData.containsKey( "searchFromattingEnabled" ) )
        {
            searchFromattingEnabled =
                searchEnabled && Boolean.parseBoolean( tableConfigData.get( "searchFromattingEnabled" ) );
        }

        if ( tableConfigData.containsKey( "searchImmediateEnabled" ) )
        {
            searchImmediateEnabled =
                searchEnabled && Boolean.parseBoolean( tableConfigData.get( "searchImmediateEnabled" ) );
        }

        if ( tableConfigData.containsKey( "searchVisibleAtStartupEnabled" ) )
        {
            searchVisibleAtStartupEnabled =
                searchEnabled && ( Boolean.parseBoolean( tableConfigData.get( "searchVisibleAtStartupEnabled" ) )
                    || !Boolean.parseBoolean( tableConfigData.get( "searchHideEnabled" ) ) );
        }

        if ( tableConfigData.containsKey( "searchHideEnabled" ) )
        {
            searchHideEnabled = searchEnabled && Boolean.parseBoolean( tableConfigData.get( "searchHideEnabled" ) );
        }

        if ( tableConfigData.containsKey( "freezeUnfreezeColumnsEnabled" ) )
        {
            freezeUnfreezeColumnsEnabled =
                Boolean.parseBoolean( tableConfigData.get( "freezeUnfreezeColumnsEnabled" ) );
        }

        if ( tableConfigData.containsKey( "freezeUnfreezeColumnsToolEnabled" ) )
        {
            freezeUnfreezeColumnsToolEnabled =
                Boolean.parseBoolean( tableConfigData.get( "freezeUnfreezeColumnsToolEnabled" ) );
        }

        if ( tableConfigData.containsKey( "freezeAnyColumnsEnabled" ) )
        {
            freezeAnyColumnsEnabled =
                frozenTableEnabled && Boolean.parseBoolean( tableConfigData.get( "freezeAnyColumnsEnabled" ) );
        }

        if ( tableConfigData.containsKey( "unfreezeAnyColumnsEnabled" ) )
        {
            unfreezeAnyColumnsEnabled =
                frozenTableEnabled && Boolean.parseBoolean( tableConfigData.get( "unfreezeAnyColumnsEnabled" ) );
        }

        if ( tableConfigData.containsKey( "hideUnhideColumnsEnabled" ) )
        {
            hideUnhideColumnsEnabled = Boolean.parseBoolean( tableConfigData.get( "hideUnhideColumnsEnabled" ) );
        }

        if ( tableConfigData.containsKey( "hideUnhideColumnsToolEnabled" ) )
        {
            hideUnhideColumnsToolEnabled =
                Boolean.parseBoolean( tableConfigData.get( "hideUnhideColumnsToolEnabled" ) );
        }

        if ( tableConfigData.containsKey( "hideAnyColumnsEnabled" ) )
        {
            hideAnyColumnsEnabled = Boolean.parseBoolean( tableConfigData.get( "hideAnyColumnsEnabled" ) );
        }

        if ( tableConfigData.containsKey( "newDataWhenHideUnhideColumnEnabled" ) )
        {
            newDataWhenHideUnhideColumnEnabled =
                Boolean.parseBoolean( tableConfigData.get( "newDataWhenHideUnhideColumnEnabled" ) );
        }

        if ( tableConfigData.containsKey( "highlightActiveRowEnabled" ) )
        {
            highlightActiveRowEnabled = Boolean.parseBoolean( tableConfigData.get( "highlightActiveRowEnabled" ) );
        }

        if ( tableConfigData.containsKey( "oddAndEvenColoringRowsEnabled" ) )
        {
            oddAndEvenColoringRowsEnabled =
                Boolean.parseBoolean( tableConfigData.get( "oddAndEvenColoringRowsEnabled" ) );
        }

        if ( tableConfigData.containsKey( "wordWrapInHeaderCellsEnabled" ) )
        {
            wordWrapInHeaderCellsEnabled =
                Boolean.parseBoolean( tableConfigData.get( "wordWrapInHeaderCellsEnabled" ) );
        }

        if ( tableConfigData.containsKey( "wordWrapInBodyCellsEnabled" ) )
        {
            wordWrapInBodyCellsEnabled = Boolean.parseBoolean( tableConfigData.get( "wordWrapInBodyCellsEnabled" ) );
        }

        if ( tableConfigData.containsKey( "expandedFrozenHeaderCellEnabled" ) )
        {
            expandedFrozenHeaderCellEnabled =
                frozenTableEnabled && Boolean.parseBoolean( tableConfigData.get( "expandedFrozenHeaderCellEnabled" ) );
            if ( expandedFrozenHeaderCellEnabled
                && tableConfigData.containsKey( "expandedFrozenHeaderCellAlwaysVisibleEnabled" ) )
            {
                expandedFrozenHeaderCellAlwaysVisibleEnabled =
                    Boolean.parseBoolean( tableConfigData.get( "expandedFrozenHeaderCellAlwaysVisibleEnabled" ) );
            }
        }

        if ( tableConfigData.containsKey( "expandedFrozenHeaderCellForActiveColumnEnabled" ) )
        {
            expandedFrozenHeaderCellForActiveColumnEnabled = frozenTableEnabled
                && Boolean.parseBoolean( tableConfigData.get( "expandedFrozenHeaderCellForActiveColumnEnabled" ) );
        }

        if ( tableConfigData.containsKey( "expandedFrozenBodyCellEnabled" ) )
        {
            expandedFrozenBodyCellEnabled =
                frozenTableEnabled && Boolean.parseBoolean( tableConfigData.get( "expandedFrozenBodyCellEnabled" ) );
            if ( expandedFrozenBodyCellEnabled
                && tableConfigData.containsKey( "expandedFrozenBodyCellAlwaysVisibleEnabled" ) )
            {
                expandedFrozenBodyCellAlwaysVisibleEnabled =
                    Boolean.parseBoolean( tableConfigData.get( "expandedFrozenBodyCellAlwaysVisibleEnabled" ) );
            }
        }

        if ( tableConfigData.containsKey( "expandedHeaderCellEnabled" ) )
        {
            expandedHeaderCellEnabled = Boolean.parseBoolean( tableConfigData.get( "expandedHeaderCellEnabled" ) );
            if ( expandedHeaderCellEnabled && tableConfigData.containsKey( "expandedHeaderCellAlwaysVisibleEnabled" ) )
            {
                expandedHeaderCellAlwaysVisibleEnabled =
                    Boolean.parseBoolean( tableConfigData.get( "expandedHeaderCellAlwaysVisibleEnabled" ) );
            }
        }

        if ( tableConfigData.containsKey( "expandedHeaderCellForActiveColumnEnabled" ) )
        {
            expandedHeaderCellForActiveColumnEnabled =
                Boolean.parseBoolean( tableConfigData.get( "expandedHeaderCellForActiveColumnEnabled" ) );
        }

        if ( tableConfigData.containsKey( "expandedBodyCellEnabled" ) )
        {
            expandedBodyCellEnabled = Boolean.parseBoolean( tableConfigData.get( "expandedBodyCellEnabled" ) );
            if ( expandedBodyCellEnabled && tableConfigData.containsKey( "expandedBodyCellAlwaysVisibleEnabled" ) )
            {
                expandedBodyCellAlwaysVisibleEnabled =
                    Boolean.parseBoolean( tableConfigData.get( "expandedBodyCellAlwaysVisibleEnabled" ) );
            }
        }

        if ( tableConfigData.containsKey( "horizontalLinesEnabled" ) )
        {
            horizontalLinesEnabled = Boolean.parseBoolean( tableConfigData.get( "horizontalLinesEnabled" ) );
        }

        if ( tableConfigData.containsKey( "headerVerticalLinesEnabled" ) )
        {
            headerVerticalLinesEnabled = Boolean.parseBoolean( tableConfigData.get( "headerVerticalLinesEnabled" ) );
        }

        if ( tableConfigData.containsKey( "bodyVerticalLinesEnabled" ) )
        {
            bodyVerticalLinesEnabled = Boolean.parseBoolean( tableConfigData.get( "bodyVerticalLinesEnabled" ) );
        }

        if ( tableConfigData.containsKey( "loadingMessageEnabled" ) )
        {
            loadingMessageEnabled = Boolean.parseBoolean( tableConfigData.get( "loadingMessageEnabled" ) );
        }

        if ( tableConfigData.containsKey( "pageResizingActionDelay" ) )
        {
            pageResizingActionDelay = Integer.parseInt( tableConfigData.get( "pageResizingActionDelay" ) );
        }

        if ( tableConfigData.containsKey( "scrollbarActionDelay" ) )
        {
            scrollbarActionDelay = Integer.parseInt( tableConfigData.get( "scrollbarActionDelay" ) );
        }

        if ( tableConfigData.containsKey( "tableToolsMouseClickActionDelay" ) )
        {
            tableToolsMouseClickActionDelay =
                Integer.parseInt( tableConfigData.get( "tableToolsMouseClickActionDelay" ) );
        }

        if ( tableConfigData.containsKey( "tableToolsTrayMouseOverAnimationDuration" ) )
        {
            tableToolsTrayMouseOverAnimationDuration =
                Integer.parseInt( tableConfigData.get( "tableToolsTrayMouseOverAnimationDuration" ) );
        }

        if ( tableConfigData.containsKey( "tableToolsTrayAnimationDuration" ) )
        {
            tableToolsTrayAnimationDuration =
                Integer.parseInt( tableConfigData.get( "tableToolsTrayAnimationDuration" ) );
        }

        if ( tableConfigData.containsKey( "quickSearchAnimationDuration" ) )
        {
            quickSearchAnimationDuration = Integer.parseInt( tableConfigData.get( "quickSearchAnimationDuration" ) );
        }

        if ( tableConfigData.containsKey( "searchAnimationDuration" ) )
        {
            searchAnimationDuration = Integer.parseInt( tableConfigData.get( "searchAnimationDuration" ) );
        }

        if ( tableConfigData.containsKey( "loadingMessageOpacity" ) )
        {
            loadingMessageOpacity = Float.parseFloat( tableConfigData.get( "loadingMessageOpacity" ) );
        }

        if ( tableConfigData.containsKey( "loadingMessageShowDelay" )
            && tableConfigData.containsKey( "loadingMessageHideDelay" ) )
        {
            loadingMessageShowDelay = Integer.parseInt( tableConfigData.get( "loadingMessageShowDelay" ) );
            loadingMessageHideDelay = Integer.parseInt( tableConfigData.get( "loadingMessageHideDelay" ) );
        }

        if ( tableConfigData.containsKey( "tableFrozenMinimalWidth" ) && frozenTableEnabled )
        {
            tableFrozenMinimalWidth = Integer.parseInt( tableConfigData.get( "tableFrozenMinimalWidth" ) );
        }

        if ( tableConfigData.containsKey( "contextMenuDepthForFilteredFrozenTable" ) )
        {
            contextMenuDepthForFilteredFrozenTable =
                Integer.parseInt( tableConfigData.get( "contextMenuDepthForFilteredFrozenTable" ) );
        }

        if ( tableConfigData.containsKey( "contextMenuDepthForFilteredTable" ) )
        {
            contextMenuDepthForFilteredTable =
                Integer.parseInt( tableConfigData.get( "contextMenuDepthForFilteredTable" ) );
        }

        if ( tableConfigData.containsKey( "pagerPages" ) )
        {
            pagerPages = Integer.parseInt( tableConfigData.get( "pagerPages" ) );
        }

        if ( tableConfigData.containsKey( "pageSize" ) )
        {
            pageSize = Integer.parseInt( tableConfigData.get( "pageSize" ) );
        }

        if ( tableConfigData.containsKey( "tableToolsPosition" ) )
        {
            tableToolsPosition = "-" + tableConfigData.get( "tableToolsPosition" );
        }

        if ( tableConfigData.containsKey( "filterInfoPosition" ) )
        {
            filterInfoPosition = tableConfigData.get( "filterInfoPosition" );
        }

        if ( tableConfigData.containsKey( "tableMinimalWidth" ) )
        {
            tableMinimalWidth = Integer.parseInt( tableConfigData.get( "tableMinimalWidth" ) );
        }

        if ( tableConfigData.containsKey( "headerRowsAlignment" ) )
        {
            headerRowsAlignment = Integer.parseInt( tableConfigData.get( "headerRowsAlignment" ) );
        }

        if ( tableConfigData.containsKey( "headerCellMarginTop" ) )
        {
            headerCellMarginTop = Integer.parseInt( tableConfigData.get( "headerCellMarginTop" ) );
        }

        if ( tableConfigData.containsKey( "headerCellMarginRight" ) )
        {
            headerCellMarginRight = Integer.parseInt( tableConfigData.get( "headerCellMarginRight" ) );
        }

        if ( tableConfigData.containsKey( "headerCellMarginBottom" ) )
        {
            headerCellMarginBottom = Integer.parseInt( tableConfigData.get( "headerCellMarginBottom" ) );
        }

        if ( tableConfigData.containsKey( "headerCellMarginLeft" ) )
        {
            headerCellMarginLeft = Integer.parseInt( tableConfigData.get( "headerCellMarginLeft" ) );
        }

        if ( tableConfigData.containsKey( "headerLineHeight" ) )
        {
            headerLineHeight = Integer.parseInt( tableConfigData.get( "headerLineHeight" ) );
        }

        // don't move
        if ( tableConfigData.containsKey( "headerRowHeight" ) )
        {
            headerRowHeight = Math.max( Integer.parseInt( tableConfigData.get( "headerRowHeight" ) ),
                                        headerLineHeight + headerCellMarginTop + headerCellMarginBottom + 1 );
        }

        if ( tableConfigData.containsKey( "bodyRowsAlignment" ) )
        {
            bodyRowAlignments = Integer.parseInt( tableConfigData.get( "bodyRowsAlignment" ) );
        }

        if ( tableConfigData.containsKey( "bodyCellMarginTop" ) )
        {
            bodyCellMarginTop = Integer.parseInt( tableConfigData.get( "bodyCellMarginTop" ) );
        }

        if ( tableConfigData.containsKey( "bodyCellMarginRight" ) )
        {
            bodyCellMarginRight = Integer.parseInt( tableConfigData.get( "bodyCellMarginRight" ) );
        }

        if ( tableConfigData.containsKey( "bodyCellMarginBottom" ) )
        {
            bodyCellMarginBottom = Integer.parseInt( tableConfigData.get( "bodyCellMarginBottom" ) );
        }

        if ( tableConfigData.containsKey( "bodyCellMarginLeft" ) )
        {
            bodyCellMarginLeft = Integer.parseInt( tableConfigData.get( "bodyCellMarginLeft" ) );
        }

        if ( tableConfigData.containsKey( "bodyLineHeight" ) )
        {
            bodyLineHeight = Integer.parseInt( tableConfigData.get( "bodyLineHeight" ) );
        }

        if ( tableConfigData.containsKey( "tooltipEnabled" ) )
        {
            tooltipEnabled = Boolean.parseBoolean( tableConfigData.get( "tooltipEnabled" ) );
        }

        if ( tableConfigData.containsKey( "wordWrapInTooltipEnabled" ) )
        {
            wordWrapInTooltipEnabled = Boolean.parseBoolean( tableConfigData.get( "wordWrapInTooltipEnabled" ) );
        }

        if ( tableConfigData.containsKey( "tableToolsMenuEnabled" ) )
        {
            tableToolsMenuEnabled = Boolean.parseBoolean( tableConfigData.get( "tableToolsMenuEnabled" ) );
        }

        if ( tableConfigData.containsKey( "tableToolsMenuAlwaysVisibleEnabled" ) )
        {
            tableToolsMenuAlwaysVisibleEnabled =
                Boolean.parseBoolean( tableConfigData.get( "tableToolsMenuAlwaysVisibleEnabled" ) );
        }

        if ( tableConfigData.containsKey( "tableToolsTrayEnabled" ) )
        {
            tableToolsTrayEnabled = Boolean.parseBoolean( tableConfigData.get( "tableToolsTrayEnabled" ) );
        }

        if ( tableConfigData.containsKey( "tooltipPosition" ) )
        {
            tooltipPosition = tableConfigData.get( "tooltipPosition" );
        }

        if ( tableConfigData.containsKey( "tooltipMinimumWidth" ) )
        {
            tooltipMinimumWidth = Math.max( 0, Integer.parseInt( tableConfigData.get( "tooltipMinimumWidth" ) ) );
        }

        if ( tableConfigData.containsKey( "tooltipMaximumWidth" ) )
        {
            tooltipMaximumWidth = Math.max( 0, Integer.parseInt( tableConfigData.get( "tooltipMaximumWidth" ) ) );
        }

        // don't move
        if ( tableConfigData.containsKey( "bodyRowHeight" ) )
        {
            bodyRowHeight = Math.max( Integer.parseInt( tableConfigData.get( "bodyRowHeight" ) ),
                                      bodyLineHeight + bodyCellMarginTop + bodyCellMarginBottom + 1 );
        }

        // don't move
        if ( tableConfigData.containsKey( "tableHeight" ) )
        {
            tableHeight = pageableTableEnabled ? bodyRowHeight * pageSize
                            : Integer.parseInt( tableConfigData.get( "tableHeight" ) ) - headerRowHeight;
        }
    }

    public int getHeaderRowsAlignment()
    {
        return headerRowsAlignment;
    }

    public int getTableFrozenMinimalWidth()
    {
        return tableFrozenMinimalWidth;
    }

    public int getTableMinimalWidth()
    {
        return tableMinimalWidth;
    }

    public int getContextMenuDepthForFilteredFrozenTable()
    {
        return contextMenuDepthForFilteredFrozenTable;
    }

    public int getContextMenuDepthForFilteredTable()
    {
        return contextMenuDepthForFilteredTable;
    }

    public int getTableHeight()
    {
        return tableHeight;
    }

    public int getPagerPages()
    {
        return pagerPages;
    }

    public int getPageSize()
    {
        return pageSize;
    }

    public String getTableToolsPosition()
    {
        return tableToolsPosition;
    }

    public boolean isFilterInfoOnTop()
    {
        return "top".equals( filterInfoPosition );
    }

    public boolean isFilterInfoOnBottom()
    {
        return "bottom".equals( filterInfoPosition );
    }

    public int getHeaderRowHeight()
    {
        return headerRowHeight;
    }

    public int getHeaderLineHeight()
    {
        return headerLineHeight;
    }

    public int getHeaderCellMarginTop()
    {
        return headerCellMarginTop;
    }

    public int getHeaderCellMarginRight()
    {
        return headerCellMarginRight;
    }

    public int getHeaderCellMarginBottom()
    {
        return headerCellMarginBottom;
    }

    public int getHeaderCellMarginLeft()
    {
        return headerCellMarginLeft;
    }

    public int getPageResizingActionDelay()
    {
        return pageResizingActionDelay;
    }

    public int getScrollbarActionDelay()
    {
        return scrollbarActionDelay;
    }

    public int getLoadingMessageShowDelay()
    {
        return loadingMessageShowDelay;
    }

    public int getLoadingMessageHideDelay()
    {
        return loadingMessageHideDelay;
    }

    public float getLoadingMessageOpacity()
    {
        return loadingMessageOpacity;
    }

    public int getBodyRowAlignments()
    {
        return bodyRowAlignments;
    }

    public int getBodyRowHeight()
    {
        return bodyRowHeight;
    }

    public int getBodyLineHeight()
    {
        return bodyLineHeight;
    }

    public int getBodyCellMarginTop()
    {
        return bodyCellMarginTop;
    }

    public int getBodyCellMarginRight()
    {
        return bodyCellMarginRight;
    }

    public int getBodyCellMarginBottom()
    {
        return bodyCellMarginBottom;
    }

    public int getBodyCellMarginLeft()
    {
        return bodyCellMarginLeft;
    }

    public int getTableToolsMouseClickActionDelay()
    {
        return tableToolsMouseClickActionDelay;
    }

    public int getTableToolsTrayMouseOverAnimationDuration()
    {
        return tableToolsTrayMouseOverAnimationDuration;
    }

    public int getTableToolsTrayAnimationDuration()
    {
        return tableToolsTrayAnimationDuration;
    }

    public int getQuickSearchAnimationDuration()
    {
        return quickSearchAnimationDuration;
    }

    public int getSearchAnimationDuration()
    {
        return searchAnimationDuration;
    }

    public boolean isColumnFormatted( int column )
    {
        return columnsFormatted.contains( column );
    }

    public void setColumnsFormatted( Object[] columns )
    {
        if ( columns != null && columns.length > 0 )
        {
            columnsFormatted.clear();
            for ( Object column : columns )
            {
                columnsFormatted.add( Integer.parseInt( column.toString() ) );
            }
        }
    }

    public boolean isColumnFreezeEnabled( int column )
    {
        return columnsFreezeEnabled.contains( "+" + column );
    }

    public boolean isColumnFreezeDisabled( int column )
    {
        return columnsFreezeEnabled.contains( "-" + column );
    }

    public void setColumnsFreezeEnabled( Object[] columns )
    {
        if ( columns != null && columns.length > 0 )
        {
            columnsFreezeEnabled.clear();
            for ( Object column : columns )
            {
                columnsFreezeEnabled.add( column.toString() );
            }
        }
    }

    public boolean isColumnUnfreezeEnabled( int column )
    {
        return columnsUnfreezeEnabled.contains( "+" + column );
    }

    public boolean isColumnUnfreezeDisabled( int column )
    {
        return columnsUnfreezeEnabled.contains( "-" + column );
    }

    public void setColumnsUnfreezeEnabled( Object[] columns )
    {
        if ( columns != null && columns.length > 0 )
        {
            columnsUnfreezeEnabled.clear();
            for ( Object column : columns )
            {
                columnsUnfreezeEnabled.add( column.toString() );
            }
        }
    }

    public boolean isColumnHideEnabled( int column )
    {
        if ( columnsHideEnabled.contains( "-" + column ) )
        {
            return false;
        }
        else if ( columnsHideEnabled.contains( "+" + column ) )
        {
            return true;
        }
        return hideAnyColumnsEnabled;
    }

    public void setColumnsHideEnabled( Object[] columns )
    {
        if ( columns != null && columns.length > 0 )
        {
            columnsHideEnabled.clear();
            for ( Object column : columns )
            {
                columnsHideEnabled.add( column.toString() );
            }
        }
    }

    public boolean isAllHeaderColumnsEnabled()
    {
        return allHeaderColumnsEnabled;
    }

    public boolean isFixedTableHeightEnabled()
    {
        return fixedTableHeightEnabled;
    }

    public boolean isAllColumnsEnabled()
    {
        return allColumnsEnabled;
    }

    public boolean isAllRowsEnabled()
    {
        return allRowsEnabled;
    }

    public boolean isQuickSearchEnabled()
    {
        return quickSearchEnabled;
    }

    public boolean isQuickSearchImmediateEnabled()
    {
        return quickSearchImmediateEnabled;
    }

    public boolean isQuickSearchVisibleAtStartupEnabled()
    {
        return quickSearchVisibleAtStartupEnabled;
    }

    public boolean isQuickSearchHideEnabled()
    {
        return quickSearchHideEnabled;
    }

    public boolean isSearchEnabled()
    {
        return searchEnabled;
    }

    public boolean isSearchFromattingEnabled()
    {
        return searchFromattingEnabled;
    }

    public boolean isSearchImmediateEnabled()
    {
        return searchImmediateEnabled;
    }

    public boolean isSearchVisibleAtStartupEnabled()
    {
        return searchVisibleAtStartupEnabled;
    }

    public boolean isSearchHideEnabled()
    {
        return searchHideEnabled;
    }

    public boolean isPageableTableEnabled()
    {
        return pageableTableEnabled;
    }

    public boolean isFrozenTableEnabled()
    {
        return frozenTableEnabled;
    }

    public boolean isFrozenTableVisibleEnabled()
    {
        return frozenTableVisibleEnabled;
    }

    public boolean isHideFrozenTableEnabled()
    {
        return hideFrozenTableEnabled;
    }

    public boolean isResizeFrozenTableEnabled()
    {
        return resizeFrozenTableEnabled;
    }

    public boolean isEmptyFrozenTableEnabled()
    {
        return emptyFrozenTableEnabled;
    }

    public boolean isFreezeUnfreezeColumnsEnabled()
    {
        return freezeUnfreezeColumnsEnabled;
    }

    public boolean isFreezeUnfreezeColumnsToolEnabled()
    {
        return freezeUnfreezeColumnsToolEnabled;
    }

    public boolean isFreezeAnyColumnsEnabled()
    {
        return freezeAnyColumnsEnabled;
    }

    public boolean isUnfreezeAnyColumnsEnabled()
    {
        return unfreezeAnyColumnsEnabled;
    }

    public boolean isHideUnhideColumnsEnabled()
    {
        return hideUnhideColumnsEnabled;
    }

    public boolean isHideUnhideColumnsToolEnabled()
    {
        return hideUnhideColumnsToolEnabled;
    }

    public boolean isHideAnyColumnsEnabled()
    {
        return hideAnyColumnsEnabled;
    }

    public boolean isNewDataWhenHideUnhideColumnEnabled()
    {
        return newDataWhenHideUnhideColumnEnabled;
    }

    public boolean isFreezeEnabled( int column )
    {
        if ( isColumnFreezeEnabled( column ) )
        {
            return true;
        }
        else if ( isColumnFreezeDisabled( column ) )
        {
            return false;
        }
        return isFreezeAnyColumnsEnabled();
    }

    public boolean isUnfreezeEnabled( int column )
    {
        if ( isColumnUnfreezeEnabled( column ) )
        {
            return true;
        }
        else if ( isColumnUnfreezeDisabled( column ) )
        {
            return false;
        }
        return isUnfreezeAnyColumnsEnabled();
    }

    public boolean isHighlightActiveRowEnabled()
    {
        return highlightActiveRowEnabled;
    }

    public boolean isOrderedFrozenTableEnabled()
    {
        return orderedFrozenTableEnabled;
    }

    public boolean isOrderedTableEnabled()
    {
        return orderedTableEnabled;
    }

    public boolean isFilteredFrozenTableEnabled()
    {
        return filteredFrozenTableEnabled;
    }

    public boolean isFilteredTableEnabled()
    {
        return filteredTableEnabled;
    }

    public boolean isContextMenuForFilteredFrozenTableEnabled()
    {
        return contextMenuForFilteredFrozenTableEnabled;
    }

    public boolean isContextMenuForFilteredTableEnabled()
    {
        return contextMenuForFilteredTableEnabled;
    }

    public boolean isExpandedFrozenHeaderCellEnabled()
    {
        return expandedFrozenHeaderCellEnabled;
    }

    public boolean isExpandedFrozenHeaderCellAlwaysVisibleEnabled()
    {
        return expandedFrozenHeaderCellAlwaysVisibleEnabled;
    }

    public boolean isExpandedFrozenHeaderCellForActiveColumnEnabled()
    {
        return expandedFrozenHeaderCellForActiveColumnEnabled;
    }

    public boolean isExpandedFrozenBodyCellEnabled()
    {
        return expandedFrozenBodyCellEnabled;
    }

    public boolean isExpandedFrozenBodyCellAlwaysVisibleEnabled()
    {
        return expandedFrozenBodyCellAlwaysVisibleEnabled;
    }

    public boolean isExpandedHeaderCellEnabled()
    {
        return expandedHeaderCellEnabled;
    }

    public boolean isExpandedHeaderCellAlwaysVisibleEnabled()
    {
        return expandedHeaderCellAlwaysVisibleEnabled;
    }

    public boolean isExpandedHeaderCellForActiveColumnEnabled()
    {
        return expandedHeaderCellForActiveColumnEnabled;
    }

    public boolean isExpandedBodyCellEnabled()
    {
        return expandedBodyCellEnabled;
    }

    public boolean isExpandedBodyCellAlwaysVisibleEnabled()
    {
        return expandedBodyCellAlwaysVisibleEnabled;
    }

    public boolean isWordWrapInHeaderCellsEnabled()
    {
        return wordWrapInHeaderCellsEnabled;
    }

    public boolean isWordWrapInBodyCellsEnabled()
    {
        return wordWrapInBodyCellsEnabled;
    }

    public boolean isFilterInfoEnabled()
    {
        return filterInfoEnabled;
    }

    public boolean isOddAndEvenColoringRowsEnabled()
    {
        return oddAndEvenColoringRowsEnabled;
    }

    public boolean isHorizontalLinesEnabled()
    {
        return horizontalLinesEnabled;
    }

    public boolean isHeaderVerticalLinesEnabled()
    {
        return headerVerticalLinesEnabled;
    }

    public boolean isBodyVerticalLinesEnabled()
    {
        return bodyVerticalLinesEnabled;
    }

    public boolean isLoadingMessageEnabled()
    {
        return loadingMessageEnabled;
    }

    public boolean isTooltipEnabled()
    {
        return tooltipEnabled;
    }

    public boolean isWordWrapInTooltipEnabled()
    {
        return wordWrapInTooltipEnabled;
    }

    public boolean isTableToolsMenuEnabled()
    {
        return tableToolsMenuEnabled;
    }

    public boolean isTableToolsMenuAlwaysVisibleEnabled()
    {
        return tableToolsMenuAlwaysVisibleEnabled;
    }

    public boolean isTableToolsTrayEnabled()
    {
        return tableToolsTrayEnabled;
    }

    public String getTooltipPosition()
    {
        return tooltipPosition;
    }

    public int getTooltipMinimumWidth()
    {
        return tooltipMinimumWidth;
    }

    public int getTooltipMaximumWidth()
    {
        return tooltipMaximumWidth;
    }

}
