/**
 * Copyright (C) 2012-2016  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client.components;

import static com.google.gwt.query.client.GQuery.$;

import org.dussan.vaadin.dtable.client.VDTable;
import org.dussan.vaadin.dtable.client.components.base.TablePanelComponent;

import com.google.gwt.dom.client.Element;
import com.google.gwt.dom.client.Style.Display;
import com.google.gwt.query.client.Function;
import com.google.gwt.query.client.GQuery;

public class Pager
    extends TablePanelComponent
{

    private String mainId = "div#v-dtable";

    private String[] filterPP = { "li#fp", "li#pps", "li#pp" };

    private String[] filterNP = { "li#np", "li#nps", "li#lp" };

    private int allPages = 0;

    private int curentPage = 1;

    private int firstPage = 1;

    private int lastPage = 5;

    private int trayPages = lastPage;

    private VDTable dTable = null;

    private GQuery pagerClickEventHandler;

    public Pager( VDTable dTable )
    {
        setStyleName( "dt-pager" );
        getElement().setId( "dt-pager" );
        this.dTable = dTable;
    }

    private void activatePagerClickEventHandler( boolean activate )
    {
        if ( activate )
        {
            GQuery pagerDiv = $( mainId + " div#dt-pager" );

            activatePagerClickEventHandler( false );
            pagerClickEventHandler = pagerDiv.on( "click", "li", new Function()
            {
                public void f( Element e )
                {
                    if ( "fp".equals( e.getId() ) )
                    {
                        // first page
                        setCurentPage( 1 );
                    }
                    else if ( "pps".equals( e.getId() ) )
                    {
                        // previous pages
                        setCurentPage( getCurentPage() - getTrayPages() );
                    }
                    else if ( "pp".equals( e.getId() ) )
                    {
                        // previous page
                        setCurentPage( getCurentPage() - 1 );
                    }
                    else if ( "np".equals( e.getId() ) )
                    {
                        // next page
                        setCurentPage( getCurentPage() + 1 );
                    }
                    else if ( "nps".equals( e.getId() ) )
                    {
                        // next pages
                        setCurentPage( getCurentPage() + getTrayPages() );
                    }
                    else if ( "lp".equals( e.getId() ) )
                    {
                        // last page
                        setCurentPage( allPages );
                    }
                    else if ( e.getId() != null && e.getId()
                                                    .startsWith( "page" ) )
                    {
                        // current num page
                        setCurentPage( Integer.parseInt( e.getId()
                                                          .substring( "page".length() ) ) );
                    }
                    checkButtons( true );
                }
            } );
        }
        else if ( pagerClickEventHandler != null )
        {
            pagerClickEventHandler.off( "click", "li" );
            pagerClickEventHandler = null;
        }
    }

    private void checkButtons( boolean sendMessage )
    {
        if ( getCurentPage() < getFirstPage() )
        {
            setFirstPage( getCurentPage() );
            setLastPage( getFirstPage() + getTrayPages() - 1 );
            createNavigationButtons( getFirstPage(), getLastPage(), sendMessage );
        }
        else if ( getCurentPage() > getLastPage() )
        {
            setLastPage( getCurentPage() );
            setFirstPage( getCurentPage() - getTrayPages() + 1 );
            createNavigationButtons( getFirstPage(), getLastPage(), sendMessage );
        }
        else
        {
            activateNumberButton( getCurentPage(), sendMessage );
            activatePrevoiusButtons( 1 < getCurentPage() );
            activateNextButtons( getCurentPage() < allPages );
        }
    }

    private void show( boolean show )
    {
        if ( !show && isVisible() )
        {
            getElement().getStyle()
                        .setDisplay( Display.NONE );
            activatePagerClickEventHandler( false );
        }
        else if ( show && !isVisible() )
        {
            getElement().getStyle()
                        .clearDisplay();
            activatePagerClickEventHandler( true );
        }
    }

    private void createNavigationButtons( int firstPage, int lastPage, boolean sendMessage )
    {
        GQuery buttonsBox = $( "<ul/>" );

        // page navigation: previous
        if ( !( allPages < trayPages ) )
        {
            $( "<span>&#xf049</span>" ).appendTo( $( "<li/>" ).id( "fp" )
                                                              .addClass( "char-active" )
                                                              .appendTo( buttonsBox ) );
        }
        $( "<li/>" ).addClass( "char-space" )
                    .appendTo( buttonsBox );
        $( "<span>&#xf048</span>" ).appendTo( $( "<li/>" ).id( "pps" )
                                                          .addClass( "char-active" )
                                                          .appendTo( buttonsBox ) );
        $( "<li/>" ).addClass( "char-space" )
                    .appendTo( buttonsBox );
        $( "<span>&#xf053</span>" ).appendTo( $( "<li/>" ).id( "pp" )
                                                          .addClass( "char-active" )
                                                          .appendTo( buttonsBox ) );

        // page navigation: nums
        for ( int page = firstPage; page <= lastPage; page++ )
        {
            $( "<li/>" ).id( "page" + page )
                        .text( Integer.toString( page ) )
                        .addClass( "num" )
                        .appendTo( buttonsBox );
        }

        // page navigation: next
        $( "<span>&#xf054</span>" ).appendTo( $( "<li/>" ).id( "np" )
                                                          .addClass( "char-active" )
                                                          .appendTo( buttonsBox ) );
        $( "<li/>" ).addClass( "char-space" )
                    .appendTo( buttonsBox );
        $( "<span>&#xf051</span>" ).appendTo( $( "<li/>" ).id( "nps" )
                                                          .addClass( "char-active" )
                                                          .appendTo( buttonsBox ) );
        $( "<li/>" ).addClass( "char-space" )
                    .appendTo( buttonsBox );
        if ( !( allPages < trayPages ) )
        {
            $( "<span>&#xf050</span>" ).appendTo( $( "<li/>" ).id( "lp" )
                                                              .addClass( "char-active" )
                                                              .appendTo( buttonsBox ) );
        }

        $( getElement() ).empty()
                         .append( buttonsBox );
        activateNumberButton( getCurentPage(), sendMessage );
        activatePrevoiusButtons( 1 < getCurentPage() );
        activateNextButtons( getCurentPage() < allPages );
    }

    public void reset()
    {
        if ( dTable.config()
                   .isPageableTableEnabled() )
        {
            setCurentPage( 1 );
            checkButtons( false );
        }
    }

    public int getAllPages()
    {
        return allPages;
    }

    protected void setAllPages( int firstPage, int currentPage, int allPages )
    {
        this.allPages = allPages;
        if ( 0 < allPages )
        {
            setFirstPage( firstPage );
            setLastPage( getFirstPage() + getTrayPages() - 1 );
            setCurentPage( currentPage );
            createNavigationButtons( getFirstPage(), getLastPage(), false );
        }
    }

    public void setAllPages( int allRows )
    {
        if ( dTable.config()
                   .isPageableTableEnabled() )
        {
            int currentAllPages = allRows / dTable.config()
                                                  .getPageSize()
                + ( 0 < allRows % dTable.config()
                                        .getPageSize() ? 1 : 0 );
            if ( allPages != currentAllPages )
            {
                allPages = currentAllPages;
                if ( 0 < currentAllPages )
                {
                    setFirstPage( 1 );
                    setLastPage( getFirstPage() + getTrayPages() - 1 );
                    setCurentPage( 1 );
                    createNavigationButtons( getFirstPage(), getLastPage(), false );
                }
                show( 1 < currentAllPages );
            }
        }
    }

    private int getTrayPages()
    {
        return allPages < trayPages ? allPages : trayPages;
    }

    public void setTrayPages( int trayPages )
    {
        this.trayPages = trayPages <= 0 ? 5 : trayPages;
    }

    protected int getFirstPage()
    {
        return firstPage;
    }

    private void setFirstPage( int firstPage )
    {
        this.firstPage = firstPage < 1 ? 1 : firstPage;
    }

    private int getLastPage()
    {
        return lastPage;
    }

    private void setLastPage( int lastPage )
    {
        this.lastPage = allPages < lastPage ? allPages : lastPage;
    }

    public int getCurentPage()
    {
        return curentPage;
    }

    private void setCurentPage( int curentPage )
    {
        this.curentPage = curentPage < 1 ? 1 : ( allPages < curentPage ? allPages : curentPage );
    }

    private void activatePrevoiusButtons( boolean activate )
    {
        $( mainId + " div#dt-pager ul" ).children( filterPP )
                                        .removeClass( "char-active", "char" )
                                        .addClass( activate ? "char-active" : "char" );
    }

    private void activateNextButtons( boolean activate )
    {
        $( mainId + " div#dt-pager ul" ).children( filterNP )
                                        .removeClass( "char-active", "char" )
                                        .addClass( activate ? "char-active" : "char" );
    }

    private void activateNumberButton( int currentPage, boolean sendMessage )
    {
        boolean isActive = $( mainId + " div#dt-pager ul" ).children( "li#page" + currentPage )
                                                           .hasClass( "num-active" );

        $( mainId + " div#dt-pager ul" ).children( "li.num-active" )
                                        .toggleClass( "num-active", "num" );
        $( mainId + " div#dt-pager ul" ).children( "li#page" + currentPage )
                                        .toggleClass( "num-active", "num" );

        if ( !isActive && sendMessage )
        {
            dTable.sendMessage( "newData", Boolean.TRUE.toString(), true );
        }
    }

    @Override
    public void setIds()
    {
        if ( !mainId.endsWith( Long.toString( dTable.getId() ) ) )
        {
            mainId += dTable.getId();
        }
    }

    @Override
    public void load()
    {
        setIds();
        if ( dTable.config()
                   .isPageableTableEnabled() )
        {
            dTable.getElement()
                  .appendChild( this.getElement() );
            setTrayPages( dTable.config()
                                .getPagerPages() );
        }
        setEventsHandlerEnabled();
    }

    @Override
    public void unload()
    {
        setEventsHandlerDisabled();
    }

    @Override
    public void setEventsHandlerEnabled()
    {
        activatePagerClickEventHandler( dTable.config()
                                              .isPageableTableEnabled() );
    }

    @Override
    public void setEventsHandlerDisabled()
    {
        activatePagerClickEventHandler( false );
    }

}
