/**
 * Copyright (C) 2012-2016  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client.components;

import static com.google.gwt.query.client.GQuery.$;
import static com.google.gwt.query.client.plugins.Effects.Effects;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.dussan.vaadin.dtable.client.VDTable;
import org.dussan.vaadin.dtable.client.components.base.TableComponent;
import org.dussan.vaadin.dtable.client.components.events.SearchEvent;

import com.google.gwt.dom.client.Element;
import com.google.gwt.query.client.Function;
import com.google.gwt.query.client.GQuery;
import com.google.gwt.query.client.css.CSS;
import com.google.gwt.query.client.css.Length;
import com.google.gwt.user.client.Event;

public class Search
    extends TableComponent
{

    private boolean hidden = true;

    private boolean messageSend = false;

    private boolean show = false;

    private boolean showAtStartup = false;

    private int markedColumn = -1;

    private String eventValue = new String();

    private String focusValue = new String();

    private String frozenHeaderId = "table#dt-frozen-header-table";

    private String headerId = "table#dt-header-table";

    private String initValue = new String();

    private String markedValue = new String();

    private Map<Integer, String> searches = new HashMap<>();

    private GQuery focusInFrozenHeaderEventHandler = null;

    private GQuery focusInHeaderEventHandler = null;

    private GQuery focusOutFrozenHeaderEventHandler = null;

    private GQuery focusOutHeaderEventHandler = null;

    private GQuery frozenHeaderMouseOverEventHandler = null;

    private GQuery frozenHeaderKeydownEventHandler = null;

    private GQuery frozenHeaderKeyupEventHandler = null;

    private GQuery frozenInputMouseOverEventHandler = null;

    private GQuery headerMouseOverEventHandler = null;

    private GQuery headerKeyDownEventHandler = null;

    private GQuery headerKeyUpEventHandler = null;

    private GQuery inputMouseOverEventHandler = null;

    public Search( VDTable dTable )
    {
        super( dTable );
    }

    private Search getThis()
    {
        return this;
    }

    private void activateFrozenInputMouseOverEventHandler( boolean activate )
    {
        if ( activate )
        {
            activateFrozenInputMouseOverEventHandler( false );

            GQuery header = $( frozenHeaderId ).children( "tr" )
                                               .eq( 1 )
                                               .children( "td" )
                                               .children( "div" );
            frozenInputMouseOverEventHandler = header.on( "mouseover", "input", new Function()
            {
                public void f( Element e )
                {
                    SearchEvent.mouseOver( getThis(), frozenHeaderId, $( e ).id() );
                }
            } )
                                                     .on( "mouseout", "input", new Function()
                                                     {
                                                         public void f( Element e )
                                                         {
                                                             SearchEvent.mouseOut( getThis(), frozenHeaderId,
                                                                                   $( e ).id() );
                                                         }
                                                     } );
        }
        else if ( frozenInputMouseOverEventHandler != null )
        {
            frozenInputMouseOverEventHandler.off( "mouseover", "input" )
                                            .off( "mouseout", "input" );
            frozenInputMouseOverEventHandler = null;
        }
    }

    private void activateFrozenHeaderMouseOverEventHandler( boolean activate )
    {
        if ( activate )
        {
            activateFrozenHeaderMouseOverEventHandler( false );

            GQuery header = $( frozenHeaderId ).children( "tr" )
                                               .eq( 1 );
            frozenHeaderMouseOverEventHandler = header.on( "mouseover", "td", new Function()
            {
                public void f( Element e )
                {
                    SearchEvent.mouseOver( getThis(), frozenHeaderId, $( e ).id() );
                }
            } )
                                                      .on( "mouseout", "td", new Function()
                                                      {
                                                          public void f( Element e )
                                                          {
                                                              SearchEvent.mouseOut( getThis(), frozenHeaderId,
                                                                                    $( e ).id() );
                                                          }
                                                      } );
        }
        else if ( frozenHeaderMouseOverEventHandler != null )
        {
            frozenHeaderMouseOverEventHandler.off( "mouseover", "td" )
                                             .off( "mouseout", "td" );
            frozenHeaderMouseOverEventHandler = null;
        }
    }

    private void activateFocusInFrozenHeaderEventHandler( boolean activate )
    {
        if ( activate )
        {
            activateFocusInFrozenHeaderEventHandler( false );

            GQuery header = $( frozenHeaderId ).children( "tr" )
                                               .eq( 1 );
            focusInFrozenHeaderEventHandler = header.on( "focusin", "input", new Function()
            {
                public boolean f( Event event )
                {
                    return SearchEvent.focusIn( getThis(), event );
                }
            } );
        }
        else if ( focusInFrozenHeaderEventHandler != null )
        {
            focusInFrozenHeaderEventHandler.off( "focusin", "input" );
            focusInFrozenHeaderEventHandler = null;
        }
    }

    private void activateFocusOutFrozenHeaderEventHandler( boolean activate )
    {
        if ( activate )
        {
            activateFocusOutFrozenHeaderEventHandler( false );

            GQuery header = $( frozenHeaderId ).children( "tr" )
                                               .eq( 1 );
            focusOutFrozenHeaderEventHandler = header.on( "focusout", "input", new Function()
            {
                public boolean f( Event event )
                {
                    return SearchEvent.focusOut( getThis(), event );
                }
            } );
        }
        else if ( focusOutFrozenHeaderEventHandler != null )
        {
            focusOutFrozenHeaderEventHandler.off( "focusout", "input" );
            focusOutFrozenHeaderEventHandler = null;
        }
    }

    private void activateFrozenHeaderKeydownEventHandler( boolean activate )
    {
        if ( activate )
        {
            activateFrozenHeaderKeydownEventHandler( false );

            GQuery header = $( frozenHeaderId ).children( "tr" )
                                               .eq( 1 );
            frozenHeaderKeydownEventHandler = header.on( "keydown", "input", new Function()
            {
                public boolean f( Event event )
                {
                    return SearchEvent.keyDown( getThis(), event );
                }
            } );
        }
        else if ( frozenHeaderKeydownEventHandler != null )
        {
            frozenHeaderKeydownEventHandler.off( "keydown", "input" );
            frozenHeaderKeydownEventHandler = null;
        }
    }

    private void activateFrozenHeaderKeyupEventHandler( boolean activate )
    {
        if ( activate )
        {
            activateFrozenHeaderKeyupEventHandler( false );

            GQuery header = $( frozenHeaderId ).children( "tr" )
                                               .eq( 1 );
            frozenHeaderKeyupEventHandler = header.on( "keyup", "input", new Function()
            {
                public boolean f( Event event )
                {
                    return SearchEvent.keyUp( getThis(), event );
                }
            } );
        }
        else if ( frozenHeaderKeyupEventHandler != null )
        {
            frozenHeaderKeyupEventHandler.off( "keyup", "input" );
            frozenHeaderKeyupEventHandler = null;
        }
    }

    private void activateInputMouseOverEventHandler( boolean activate )
    {
        if ( activate )
        {
            activateInputMouseOverEventHandler( false );

            GQuery header = $( headerId ).children( "tr" )
                                         .eq( 1 );
            inputMouseOverEventHandler = header.on( "mouseover", "input", new Function()
            {
                public void f( Element e )
                {
                    SearchEvent.mouseOver( getThis(), headerId, $( e ).id() );
                }
            } )
                                               .on( "mouseout", "input", new Function()
                                               {
                                                   public void f( Element e )
                                                   {
                                                       SearchEvent.mouseOut( getThis(), headerId, $( e ).id() );
                                                   }
                                               } );
        }
        else if ( inputMouseOverEventHandler != null )
        {
            inputMouseOverEventHandler.off( "mouseover", "input" )
                                      .off( "mouseout", "input" );
            inputMouseOverEventHandler = null;
        }
    }

    private void activateHeaderMouseOverEventHandler( boolean activate )
    {
        if ( activate )
        {
            activateHeaderMouseOverEventHandler( false );

            GQuery header = $( headerId ).children( "tr" )
                                         .eq( 1 );
            headerMouseOverEventHandler = header.on( "mouseover", "td", new Function()
            {
                public void f( Element e )
                {
                    SearchEvent.mouseOver( getThis(), headerId, $( e ).id() );
                }
            } )
                                                .on( "mouseout", "td", new Function()
                                                {
                                                    public void f( Element e )
                                                    {
                                                        SearchEvent.mouseOut( getThis(), headerId, $( e ).id() );
                                                    }
                                                } );
        }
        else if ( headerMouseOverEventHandler != null )
        {
            headerMouseOverEventHandler.off( "mouseover", "td" )
                                       .off( "mouseout", "td" );
            headerMouseOverEventHandler = null;
        }
    }

    private void activateFocusInHeaderEventHandler( boolean activate )
    {
        if ( activate )
        {
            activateFocusInHeaderEventHandler( false );

            GQuery header = $( headerId ).children( "tr" )
                                         .eq( 1 );
            focusInHeaderEventHandler = header.on( "focusin", "input", new Function()
            {
                public boolean f( Event event )
                {
                    return SearchEvent.focusIn( getThis(), event );
                }
            } );
        }
        else if ( focusInHeaderEventHandler != null )
        {
            focusInHeaderEventHandler.off( "focusin", "input" );
            focusInHeaderEventHandler = null;
        }
    }

    private void activateFocusOutHeaderEventHandler( boolean activate )
    {
        if ( activate )
        {
            activateFocusOutHeaderEventHandler( false );

            GQuery header = $( headerId ).children( "tr" )
                                         .eq( 1 );
            focusOutHeaderEventHandler = header.on( "focusout", "input", new Function()
            {
                public boolean f( Event event )
                {
                    return SearchEvent.focusOut( getThis(), event );
                }
            } );
        }
        else if ( focusOutHeaderEventHandler != null )
        {
            focusOutHeaderEventHandler.off( "focusout", "input" );
            focusOutHeaderEventHandler = null;
        }
    }

    private void activateHeaderKeyDownEventHandler( boolean activate )
    {
        if ( activate )
        {
            activateHeaderKeyDownEventHandler( false );

            GQuery header = $( headerId ).children( "tr" )
                                         .eq( 1 );
            headerKeyDownEventHandler = header.on( "keydown", "input", new Function()
            {
                public boolean f( Event event )
                {
                    return SearchEvent.keyDown( getThis(), event );
                }
            } );
        }
        else if ( headerKeyDownEventHandler != null )
        {
            headerKeyDownEventHandler.off( "keydown", "input" );
            headerKeyDownEventHandler = null;
        }
    }

    private void activateHeaderKeyUpEventHandler( boolean activate )
    {
        if ( activate )
        {
            activateHeaderKeyUpEventHandler( false );

            GQuery header = $( headerId ).children( "tr" )
                                         .eq( 1 );
            headerKeyUpEventHandler = header.on( "keyup", "input", new Function()
            {
                public boolean f( Event event )
                {
                    return SearchEvent.keyUp( getThis(), event );
                }
            } );
        }
        else if ( headerKeyUpEventHandler != null )
        {
            headerKeyUpEventHandler.off( "keyup", "input" );
            headerKeyUpEventHandler = null;
        }
    }

    private void removeRows( boolean animate )
    {
        if ( animate )
        {
            String animation = "height:0px;margin:0px";
            GQuery header = $( headerId ).children( "tr" )
                                         .eq( 1 )
                                         .children( "td" )
                                         .children( "div" );
            final String lastId = header.last()
                                        .id();
            header.as( Effects )
                  .animate( animation, getDTable().config()
                                                  .getSearchAnimationDuration(),
                            new Function()
                            {
                                public void f( Element e )
                                {
                                    if ( e.getId()
                                          .equals( lastId ) )
                                    {
                                        $( headerId ).children( "tr" )
                                                     .eq( 1 )
                                                     .remove();
                                    }
                                }
                            } );

            if ( getDTable().config()
                            .isFrozenTableEnabled() )
            {
                GQuery frozenHeader = $( frozenHeaderId ).children( "tr" )
                                                         .eq( 1 )
                                                         .children( "td" )
                                                         .children( "div" );
                final String lastFrozenId = frozenHeader.last()
                                                        .id();
                frozenHeader.as( Effects )
                            .animate( animation, getDTable().config()
                                                            .getSearchAnimationDuration(),
                                      new Function()
                                      {
                                          public void f( Element e )
                                          {
                                              if ( e.getId()
                                                    .equals( lastFrozenId ) )
                                              {
                                                  $( frozenHeaderId ).children( "tr" )
                                                                     .eq( 1 )
                                                                     .delay( 3000 )
                                                                     .remove();
                                              }
                                          }
                                      } );
            }
        }
        else
        {
            $( headerId ).children( "tr" )
                         .eq( 1 )
                         .remove();

            if ( getDTable().config()
                            .isFrozenTableEnabled() )
            {
                $( frozenHeaderId ).children( "tr" )
                                   .eq( 1 )
                                   .remove();
            }
        }
    }

    private void prepare( boolean animate )
    {
        removeRows( animate );
        $( headerId ).children( "tr" )
                     .eq( 0 )
                     .clone()
                     .id( getDTable().getId() + "h:1:t" )
                     .appendTo( $( headerId ) );
        if ( getDTable().config()
                        .isFrozenTableEnabled() )
        {
            $( frozenHeaderId ).children( "tr" )
                               .eq( 0 )
                               .clone()
                               .id( getDTable().getId() + "fh:1:ft" )
                               .appendTo( $( frozenHeaderId ) );
        }
    }

    private void appendSearchRow( GQuery table, final boolean animate )
    {
        GQuery trSearch = table.children( "tr" )
                               .eq( 1 );
        final String idSearch = trSearch.id()
                                        .substring( 0, trSearch.id()
                                                               .lastIndexOf( ":" )
                                            + 1 );
        GQuery td = table.children( "tr" )
                         .eq( 1 )
                         .children( "td" );
        int height = td.children( "div" )
                       .height();
        final int marginTop = getDTable().config()
                                         .getHeaderCellMarginTop();
        final int marginBottom = getDTable().config()
                                            .getHeaderCellMarginBottom();

        td.each( new Function()
        {
            public void f( Element e )
            {
                int column = Integer.parseInt( e.getId()
                                                .split( ":" )[2] );
                String id = idSearch.replace( "fh:", "h:" ) + column;
                $( e ).id( id )
                      .removeClass( "asc", "desc" )
                      .addClass( "search" )
                      .css( CSS.TEXT_ALIGN, "" )
                      .children( "div" )
                      .id( id + ":1" )
                      .removeClass( "nowrap", "normal" )
                      .css( CSS.HEIGHT, animate ? "0px" : "" )
                      .css( CSS.MARGIN_LEFT.with( Length.px( getDTable().config()
                                                                        .getHeaderCellMarginLeft() ) ) )
                      .css( CSS.MARGIN_RIGHT.with( Length.px( getDTable().config()
                                                                         .getHeaderCellMarginRight() ) ) )
                      .css( CSS.MARGIN_TOP.with( Length.px( animate ? 0 : marginTop ) ) )
                      .css( CSS.MARGIN_BOTTOM.with( Length.px( animate ? 0 : marginBottom ) ) )
                      .children( "span" )
                      .replaceWith( $( "<input/>" ).id( id + ":2" ) );

                if ( ( !getDTable().isFrozen( column ) && !getDTable().config()
                                                                      .isFilteredTableEnabled() )
                    || ( getDTable().isFrozen( column ) && !getDTable().config()
                                                                       .isFilteredFrozenTableEnabled() ) )
                {
                    $( e ).children( "div" )
                          .children( "input" )
                          .addClass( "disabled" )
                          .attr( "readonly", "readonly" );
                }
            }
        } );

        if ( animate )
        {
            String animation =
                "height:" + height + "px;margin-top:" + marginTop + "px;margin-bottom:" + marginBottom + "px";
            td.children( "div" )
              .as( Effects )
              .animate( animation, getDTable().config()
                                              .getSearchAnimationDuration(),
                        new Function()
                        {
                            public void f( Element e )
                            {
                                $( e ).css( CSS.HEIGHT, "" )
                                      .css( CSS.MARGIN_TOP.with( Length.px( marginTop ) ) )
                                      .css( CSS.MARGIN_BOTTOM.with( Length.px( marginBottom ) ) );
                            }
                        } );
        }
    }

    private void appendSearchRows( boolean animate )
    {
        appendSearchRow( $( headerId ), animate );
        if ( getDTable().config()
                        .isFrozenTableEnabled() )
        {
            appendSearchRow( $( frozenHeaderId ), animate );
        }
    }

    private void hideRows()
    {
        if ( getDTable().config()
                        .isSearchEnabled() )
        {
            removeRows( true );
            setEventsHandlerDisabled();
        }
    }

    private void showRows( boolean animate )
    {
        if ( getDTable().config()
                        .isSearchEnabled()
            && !isHidden() )
        {
            prepare( animate );
            appendSearchRows( animate );
            setEventsHandlerEnabled();
            show = false;
        }
    }

    private void setSearches( Object[] searches )
    {
        if ( searches != null && searches.length > 0 )
        {
            for ( int search = 0; search < searches.length; search += 3 )
            {
                String value = searches[search + ( getDTable().config()
                                                              .isSearchFromattingEnabled() ? 1 : 2 )].toString();
                this.searches.put( Integer.parseInt( searches[search].toString() ), value );
            }
        }
    }

    private void fillSearches()
    {
        if ( getDTable().config()
                        .isSearchEnabled()
            && isVisible() && !isHidden() )
        {
            String idFocus = $( "*:focus" ).attr( "id" );
            for ( Entry<Integer, String> data : searches.entrySet() )
            {
                Integer column = data.getKey();
                String header = ( getDTable().isFrozen( column ) ? frozenHeaderId : headerId ) + " tr";
                String id = "td [id='" + getDTable().getId() + ":1:" + column + "']";
                String idInput = getDTable().getId() + ":1:" + column + ":2";
                if ( !idInput.equals( idFocus ) )
                {
                    String searchValue = data.getValue();
                    $( header ).eq( 1 )
                               .children( id )
                               .children( "div" )
                               .children( "input" )
                               .val( setFieldValue( searchValue ) );
                }
            }
            unmarkSearchField();
        }
    }

    private void removeSearches()
    {
        if ( !isEmpty() )
        {
            getDTable().sendMessage( "search", "none", true );
        }
    }

    private void unmarkSearchField()
    {
        if ( 0 <= markedColumn )
        {
            focusValue = isSearched( markedColumn ) ? searches.get( markedColumn ) : new String();
            if ( !getDTable().config()
                             .isSearchImmediateEnabled() )
            {
                String header = ( getDTable().isFrozen( markedColumn ) ? frozenHeaderId : headerId ) + " tr";
                String id = "td [id='" + getDTable().getId() + ":1:" + markedColumn + "']";
                $( header ).eq( 1 )
                           .children( id )
                           .children( "div" )
                           .children( "input" )
                           .blur();
            }
            resetMarkedValue();
        }
    }

    private String getOperator( int column )
    {
        if ( getSearchValue( column ).startsWith( "LIKE " ) )
        {
            return "LIKE";
        }
        else if ( getSearchValue( column ).startsWith( "NOT LIKE " ) )
        {
            return "NOT LIKE";
        }
        return getSearchValue( column ).split( " " )[0].trim();
    }

    private boolean isHidden()
    {
        return hidden;
    }

    private void setHidden( boolean hidden )
    {
        this.hidden = hidden;
    }

    public boolean isVisible()
    {
        return 1 < $( headerId ).children( "tr" )
                                .size();
    }

    public void show( boolean show )
    {
        this.show = show;
    }

    public void emptySearch( int column )
    {
        markedValue = new String();
        initValue = new String();
        focusValue = new String();
        eventValue = new String();

        searches.remove( column );
        String header = ( getDTable().isFrozen( column ) ? frozenHeaderId : headerId ) + " tr";
        String id = "td [id='" + getDTable().getId() + ":1:" + column + "']";
        $( header ).eq( 1 )
                   .children( id )
                   .children( "div" )
                   .children( "input" )
                   .removeClass( "error" )
                   .val( "" )
                   .blur();
    }

    public boolean filterOperatorExists( int column, String operator )
    {
        if ( getOperator( column ).equals( operator ) )
        {
            return true;
        }
        return false;
    }

    public String setFieldValue( String value )
    {
        if ( value.startsWith( "= " ) )
        {
            return value.substring( "= ".length() );
        }
        else if ( value.startsWith( "LIKE " ) )
        {
            String tempValue = value.substring( "LIKE ".length() );
            tempValue = tempValue.startsWith( "\"" ) ? tempValue.substring( 1 ) : tempValue;
            tempValue = tempValue.startsWith( "'" ) ? tempValue.substring( 1 ) : tempValue;
            tempValue = tempValue.endsWith( "\"" ) ? tempValue.substring( 0, tempValue.length() - 1 ) : tempValue;
            tempValue = tempValue.endsWith( "'" ) ? tempValue.substring( 0, tempValue.length() - 1 ) : tempValue;
            return tempValue;
        }
        return value;
    }

    public String getFrozenHeaderId()
    {
        return frozenHeaderId;
    }

    public String getHeaderId()
    {
        return headerId;
    }

    public Boolean isMessageSend()
    {
        return messageSend;
    }

    public void setMessageSend( Boolean messageSend )
    {
        this.messageSend = messageSend;
    }

    public String getInitValue()
    {
        return initValue;
    }

    public void setInitValue( int column )
    {
        initValue = getSearchValue( column );
    }

    public String getFocusValue()
    {
        return focusValue;
    }

    public void setFocusValue( String focusValue )
    {
        this.focusValue = focusValue;
    }

    public String getEventValue()
    {
        return eventValue;
    }

    public void setEventValue( String eventValue )
    {
        this.eventValue = eventValue;
    }

    public boolean isEmpty()
    {
        return searches.isEmpty();
    }

    public boolean isSearched( int column )
    {
        return searches.containsKey( column );
    }

    public List<Integer> getSearchColumns()
    {
        return new ArrayList<>( searches.keySet() );
    }

    public String getSearchValue( int column )
    {
        if ( isSearched( column ) )
        {
            return searches.get( column );
        }
        return "";
    }

    public String getMarkedValue()
    {
        return markedValue;
    }

    public void resetMarkedValue()
    {
        markedColumn = -1;
        markedValue = new String();
    }

    public void markSearchField( int column )
    {
        String header = ( getDTable().isFrozen( column ) ? frozenHeaderId : headerId ) + " tr";
        String id = "td [id='" + getDTable().getId() + ":1:" + column + "']";
        eventValue = $( header ).eq( 1 )
                                .children( id )
                                .children( "div" )
                                .children( "input" )
                                .addClass( "error" )
                                .focus()
                                .val();
        focusValue = isSearched( column ) ? searches.get( column ) : new String();
        markedColumn = column;
        markedValue = eventValue;
    }

    public Boolean isSearchMarked()
    {
        return 0 <= markedColumn;
    }

    @Override
    public void setIds()
    {
        if ( !headerId.endsWith( Long.toString( getDTable().getId() ) ) )
        {
            frozenHeaderId += getDTable().getId();
            headerId += getDTable().getId();

            setHidden( !getDTable().config()
                                   .isSearchVisibleAtStartupEnabled() );
        }
    }

    public void processData( Map<String, String> data )
    {
        searches.clear();
        if ( data.containsKey( "search" ) )
        {
            setSearches( VDTable.toStringArray( data.get( "search" ) ) );
        }

        if ( data.containsKey( "removeFilter" ) )
        {
            emptySearch( Integer.parseInt( data.get( "removeFilter" ) ) );
        }
    }

    @Override
    public void load()
    {
        setIds();
        showAtStartup = getDTable().config()
                                   .isSearchEnabled()
            && getDTable().config()
                          .isSearchVisibleAtStartupEnabled();
    }

    @Override
    public void unload()
    {
        removeRows( false );
        setEventsHandlerDisabled();
    }

    public void apply()
    {
        if ( ( show || showAtStartup ) && !isHidden() )
        {
            showAtStartup = false;
            showRows( false );
        }
        fillSearches();
        setMessageSend( false );
    }

    public void toggle()
    {
        if ( isVisible() )
        {
            setHidden( true );
            removeSearches();
            hideRows();
        }
        else
        {
            setHidden( false );
            showRows( true );
        }
    }

    @Override
    public void setEventsHandlerEnabled()
    {
        if ( getDTable().config()
                        .isSearchEnabled()
            && ( getDTable().config()
                            .isFilteredFrozenTableEnabled()
                || getDTable().config()
                              .isFilteredTableEnabled() ) )
        {
            activateFrozenInputMouseOverEventHandler( getDTable().config()
                                                                 .isFrozenTableEnabled()
                && getDTable().config()
                              .isExpandedFrozenHeaderCellForActiveColumnEnabled() );

            activateFrozenHeaderMouseOverEventHandler( getDTable().config()
                                                                  .isFrozenTableEnabled()
                && getDTable().config()
                              .isExpandedFrozenHeaderCellForActiveColumnEnabled() );

            activateFocusInFrozenHeaderEventHandler( getDTable().config()
                                                                .isFrozenTableEnabled() );

            activateFocusOutFrozenHeaderEventHandler( getDTable().config()
                                                                 .isFrozenTableEnabled() );

            activateFrozenHeaderKeydownEventHandler( getDTable().config()
                                                                .isFrozenTableEnabled() );

            activateFrozenHeaderKeyupEventHandler( getDTable().config()
                                                              .isFrozenTableEnabled() );

            activateInputMouseOverEventHandler( getDTable().config()
                                                           .isExpandedHeaderCellForActiveColumnEnabled() );

            activateHeaderMouseOverEventHandler( getDTable().config()
                                                            .isExpandedHeaderCellForActiveColumnEnabled() );
            activateFocusInHeaderEventHandler( true );
            activateFocusOutHeaderEventHandler( true );
            activateHeaderKeyDownEventHandler( true );
            activateHeaderKeyUpEventHandler( true );
        }
    }

    @Override
    public void setEventsHandlerDisabled()
    {
        activateFrozenInputMouseOverEventHandler( false );
        activateFrozenHeaderMouseOverEventHandler( false );
        activateFocusInFrozenHeaderEventHandler( false );
        activateFocusOutFrozenHeaderEventHandler( false );
        activateFrozenHeaderKeydownEventHandler( false );
        activateFrozenHeaderKeyupEventHandler( false );
        activateInputMouseOverEventHandler( false );
        activateHeaderMouseOverEventHandler( false );
        activateFocusInHeaderEventHandler( false );
        activateFocusOutHeaderEventHandler( false );
        activateHeaderKeyDownEventHandler( false );
        activateHeaderKeyUpEventHandler( false );
    }

}
