/**
 * Copyright (C) 2012-2016  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client.components;

import static com.google.gwt.query.client.GQuery.$;

import com.google.gwt.dom.client.Element;
import com.google.gwt.query.client.css.CSS;
import com.google.gwt.query.client.css.Length;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.Window;
import com.vaadin.client.ui.VOverlay;

public class Tooltip
    extends VOverlay
{

    private boolean show = false;

    private boolean wordWrapEnabled = false;

    private int left = -1;

    private int top = -1;

    private int minWidth = 0;

    private int maxWidth = 0;

    private String text = new String();

    public Tooltip()
    {
        // Auto-generated constructor stub
    }

    public void setTo( Event e, String text, boolean wordWrapEnabled, int minWidth, int maxWidth, String postition )
    {
        if ( e != null && e.getRelatedEventTarget() != null && text != null && !text.isEmpty() )
        {
            this.text = text;
            this.wordWrapEnabled = wordWrapEnabled;
            this.minWidth = minWidth;
            this.maxWidth = maxWidth;

            Element element = Element.as( e.getRelatedEventTarget() );
            if ( 3 <= element.getId()
                             .split( ":" ).length )
            {
                String[] id = element.getId()
                                     .split( ":" );
                id[1] = "0"; // always 0 - first header row
                element = DOM.getElementById( id[0] + ":" + id[1] + ":" + id[2] );

                left = e.getClientX();
                top = element.getAbsoluteTop();
                if ( "BOTTOM".equals( postition ) )
                {
                    top += $( element ).height() + 2;
                }
            }
        }
    }

    @Override
    public void show()
    {
        if ( !show && !text.isEmpty() && 0 <= left && 0 <= top )
        {
            show = true;
            $( "<div />" ).html( text )
                          .id( "dt-tooltip" )
                          .addClass( "dt-tooltip" )
                          .addClass( wordWrapEnabled ? "normal" : "nowrap" )
                          .appendTo( $( getElement() ) );
            super.show();
            setPopupPosition( Math.min( left, Window.getClientWidth() - getOffsetWidth() - 10 ), top );

            // tooltip width
            minWidth = Math.min( minWidth, maxWidth );
            if ( 0 < minWidth )
            {
                $( "#dt-tooltip" ).css( CSS.MIN_WIDTH.with( Length.px( minWidth ) ) );
            }

            if ( 0 < maxWidth )
            {
                $( "#dt-tooltip" ).css( CSS.MAX_WIDTH.with( Length.px( maxWidth ) ) );
            }

            if ( ( Window.getClientWidth() - 30 ) < getOffsetWidth() )
            {
                $( "#dt-tooltip" ).width( Window.getClientWidth() - 30 );
            }
        }
    }

    @Override
    public void hide()
    {
        if ( show )
        {
            left = -1;
            top = -1;
            super.hide();
            getElement().removeAllChildren();
            show = false;
        }
    }

}
