/**
 * Copyright (C) 2012-2016  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client.components.events;

import static com.google.gwt.query.client.GQuery.$;

import org.dussan.vaadin.dtable.client.components.Search;

import com.google.gwt.event.dom.client.KeyCodes;
import com.google.gwt.user.client.Event;

public class SearchEvent
{

    private SearchEvent()
    {
        // Auto-generated constructor stub
    }

    private static boolean canSendSearch( Search search, int column, String searchString )
    {
        if ( "".equals( searchString.trim() ) && !searchString.isEmpty() )
        {
            return false;
        }
        else if ( search.getDTable()
                        .config()
                        .isColumnFormatted( column )
            && !"".equals( searchString.replaceAll( "\\s+", " " )
                                       .trim() ) )
        {
            String tempSearch = searchString.replaceAll( "\\s+", " " )
                                            .trim();
            while ( tempSearch.startsWith( "=" ) || tempSearch.startsWith( "<" ) || tempSearch.startsWith( ">" )
                || tempSearch.startsWith( "-" ) || tempSearch.startsWith( "+" ) || tempSearch.startsWith( "," )
                || tempSearch.startsWith( "." ) )
            {
                tempSearch = tempSearch.substring( 1 );
            }
            return 0 < tempSearch.trim()
                                 .length();
        }
        else if ( !search.getDTable()
                         .config()
                         .isColumnFormatted( column )
            && !"".equals( searchString.replaceAll( "\\s+", " " )
                                       .trim() ) )
        {
            String tempSearch = searchString.replaceAll( "\\s+", " " )
                                            .trim();
            while ( tempSearch.startsWith( "%" ) || tempSearch.startsWith( "\"" ) || tempSearch.startsWith( "'" ) )
            {
                tempSearch = tempSearch.substring( 1 );
            }
            return 0 < tempSearch.trim()
                                 .length();
        }
        return true;
    }

    private static void checkSearchField( Search search, int column, Event event )
    {
        if ( !canSendSearch( search, column, $( event ).val() ) )
        {
            $( event ).addClass( "error" );
        }
        else if ( search.isSearchMarked() && !search.getMarkedValue()
                                                    .isEmpty()
            && search.getMarkedValue()
                     .equals( $( event ).val() ) )
        {
            $( event ).addClass( "error" );
        }
        else
        {
            $( event ).removeClass( "error" );
        }
    }

    private static String prepareSearchMessage( Search search, int column, String searchString )
    {
        String tempString = searchString;
        if ( tempString != null && !tempString.trim()
                                              .isEmpty() )
        {
            tempString = searchString.replaceAll( "\\s+", " " )
                                     .trim();
            if ( search.getDTable()
                       .config()
                       .isColumnFormatted( column ) )
            {
                if ( tempString.startsWith( "<=" ) || tempString.startsWith( ">=" ) || tempString.startsWith( "<>" ) )
                {
                    tempString.replace( "<=", "<= " )
                              .replace( ">=", ">= " )
                              .replace( "<>", "<> " );
                }
                else if ( tempString.startsWith( "=" ) || tempString.startsWith( "<" ) || tempString.startsWith( ">" ) )
                {
                    tempString.replace( "<", "< " )
                              .replace( ">", "> " )
                              .replace( "=", "= " );
                }
                else
                {
                    tempString = "= " + tempString;
                }
            }
            tempString = tempString.replaceAll( "\\s+", " " )
                                   .trim();
        }
        return column + "_::_" + tempString;
    }

    public static void mouseOver( Search search, String headerId, String id )
    {
        // expanded header cell
        String[] ids = id.split( ":" );
        TableEvent.cellMouseOver( search.getDTable(),
                                  $( headerId + " tr" ).children( "td [id='" + ids[0] + ":0:" + ids[2] + "']" ), null );
    }

    public static void mouseOut( Search search, String headerId, String id )
    {
        // expanded header cell
        String[] ids = id.split( ":" );
        TableEvent.headerCellMouseOut( headerId + " tr", ids[0], ids[2], null );
    }

    public static boolean keyDown( Search search, Event event )
    {
        if ( !search.getDTable()
                    .sendingMessage()
            && event.getKeyCode() == KeyCodes.KEY_TAB )
        {
            $( event ).blur();
            return false;
        }
        return true;
    }

    public static boolean keyUp( Search search, Event event )
    {
        if ( !search.getDTable()
                    .sendingMessage() )
        {
            String[] id = $( event ).id()
                                    .split( ":" );
            if ( id != null && id.length == 4 && "2".equals( id[3] ) )
            {
                int column = Integer.parseInt( id[2] );
                String value = $( event ).val();
                checkSearchField( search, column, event );

                if ( event.getKeyCode() == KeyCodes.KEY_ESCAPE )
                {
                    search.setMessageSend( true );
                    search.setFocusValue( search.getInitValue() );
                    $( event ).blur();
                }
                else if ( event.getKeyCode() == KeyCodes.KEY_ENTER )
                {
                    if ( !search.getDTable()
                                .config()
                                .isSearchImmediateEnabled()
                        && canSendSearch( search, column, value ) && !search.getFocusValue()
                                                                            .equals( value )
                        && ( !search.getEventValue()
                                    .equals( value )
                            || "".equals( search.getEventValue() ) ) )
                    {
                        search.setEventValue( value );
                        search.setMessageSend( true );
                        search.getDTable()
                              .sendMessage( "search", prepareSearchMessage( search, column, value ), true );
                    }

                    if ( !search.isSearchMarked() )
                    {
                        if ( canSendSearch( search, column, value ) )
                        {
                            if ( search.getDTable()
                                       .config()
                                       .isSearchImmediateEnabled() )
                            {
                                value = search.getSearchValue( column );
                                search.setEventValue( value );
                            }
                            search.setFocusValue( value );
                        }
                        $( event ).blur();
                    }
                }
                else if ( search.getDTable()
                                .config()
                                .isSearchImmediateEnabled()
                    && event.getKeyCode() != KeyCodes.KEY_CTRL && event.getKeyCode() != KeyCodes.KEY_ALT
                    && event.getKeyCode() != KeyCodes.KEY_LEFT && event.getKeyCode() != KeyCodes.KEY_RIGHT
                    && event.getKeyCode() != KeyCodes.KEY_UP && event.getKeyCode() != KeyCodes.KEY_DOWN
                    && canSendSearch( search, column, value ) && !search.getEventValue()
                                                                        .equals( value ) )
                {
                    search.setMessageSend( true );
                    search.setEventValue( value );
                    search.getDTable()
                          .sendMessage( "search", prepareSearchMessage( search, column, value ), true );
                }
                return false;
            }
        }
        return true;
    }

    public static boolean focusIn( Search search, Event event )
    {
        String[] id = $( event ).id()
                                .split( ":" );
        if ( id != null && id.length == 4 && "2".equals( id[3] ) )
        {
            int column = Integer.parseInt( id[2] );
            if ( ( search.getDTable()
                         .isFrozen( column )
                && !search.getDTable()
                          .config()
                          .isFilteredFrozenTableEnabled() )
                || ( !search.getDTable()
                            .isFrozen( column )
                    && !search.getDTable()
                              .config()
                              .isFilteredTableEnabled() ) )
            {
                $( event ).blur();
                return true;
            }
            search.setInitValue( column );
            search.setFocusValue( $( event ).val() );
            search.setEventValue( $( event ).val() );
            search.resetMarkedValue();
        }
        search.setMessageSend( false );

        return true;
    }

    public static boolean focusOut( Search search, Event event )
    {
        if ( $( event ).id() != null && $( event ).id()
                                                  .split( ":" ).length == 4 )
        {
            String[] id = $( event ).id()
                                    .split( ":" );
            if ( "2".equals( id[3] ) )
            {
                String value = search.getFocusValue();
                if ( !search.isMessageSend() && !value.equals( search.getEventValue() ) )
                {
                    search.setEventValue( value );
                    search.getDTable()
                          .sendMessage( "search", prepareSearchMessage( search, Integer.parseInt( id[2] ), value ),
                                        true );
                }
                $( event ).val( search.setFieldValue( value ) );
            }
            $( event ).removeClass( "error" );
        }
        return true;
    }

}
