/**
 * Copyright (C) 2012-2016  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client.components.menu;

import static com.google.gwt.query.client.GQuery.$;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.dussan.vaadin.dtable.client.VDTable;

import com.google.gwt.dom.client.Element;
import com.google.gwt.event.dom.client.KeyCodes;
import com.google.gwt.query.client.Function;
import com.google.gwt.query.client.css.CSS;
import com.google.gwt.query.client.css.Length;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.Event.NativePreviewEvent;
import com.vaadin.client.ui.VOverlay;

public class ContextMenu
    extends VOverlay
{

    private int column = 0;

    private int row = 0;

    private String cellValue = new String();

    private VDTable dTable = null;

    private Element contextMenuElement = null;

    private Element targetE = null;

    public ContextMenu( VDTable dTable )
    {
        this.dTable = dTable;

        getElement().setClassName( "dt-menu" );
        setAutoHideEnabled( true );

        if ( 0 < DOM.getChildCount( getElement() ) )
        {
            getElement().getFirstChildElement()
                        .removeFromParent();
        }

        sinkEvents( Event.ONCLICK | Event.ONCONTEXTMENU );
    }

    private VDTable getDTable()
    {
        return dTable;
    }

    public void add( Element e )
    {
        if ( contextMenuElement != null )
        {
            contextMenuElement.removeFromParent();
        }

        // glass layer to prevent browser's events
        $( "<div />" ).id( "dt-menu-glass" )
                      .addClass( "dt-menu-glass" )
                      .css( CSS.WIDTH.with( Length.px( Window.getClientWidth() ) ) )
                      .css( CSS.HEIGHT.with( Length.px( Window.getClientHeight() ) ) )
                      .on( "click", new Function()
                      {
                          public void f( Element e )
                          {
                              $( e ).remove();
                          }
                      } )
                      .appendTo( $( "body" ) );

        // context menu
        targetE = e;
        contextMenuElement = DOM.createDiv();
        getElement().appendChild( contextMenuElement );

        String[] id = e.getId()
                       .split( ":" );
        cellValue = $( e ).children( "div" )
                          .children( "span" )
                          .html()
                          .trim();
        List<ContextMenuItems> menuItems = new ArrayList<>();

        if ( 3 <= id.length && id[0].endsWith( "h" ) )
        // table header
        {
            column = Integer.parseInt( id[2] );

            // ascending/descending/remove order
            if ( getDTable().order()
                            .isOrderEnabled( column ) )
            {
                String orderType = ContextMenuItems.ORDER_REMOVE.id();
                if ( $( e ).hasClass( "asc" ) )
                {
                    orderType = ContextMenuItems.ORDER_ASCENDING.id();
                }
                else if ( $( e ).hasClass( "desc" ) )
                {
                    orderType = ContextMenuItems.ORDER_DESCENDING.id();
                }

                menuItems.addAll( Arrays.asList( ContextMenuItems.ORDER_ASCENDING, ContextMenuItems.ORDER_DESCENDING,
                                                 ContextMenuItems.ORDER_REMOVE ) );
                for ( ContextMenuItems menuItem : menuItems )
                {
                    String description = getDTable().translation()
                                                    .get( menuItem.description() );
                    if ( !menuItem.id()
                                  .equalsIgnoreCase( orderType ) )
                    {
                        Element item = DOM.createDiv();
                        item.setId( menuItem.id() );
                        item.setInnerText( description.replace( "_", cellValue ) );
                        contextMenuElement.appendChild( item );
                    }
                }

                if ( ( $( e ).parent()
                             .children( "td.asc", "td.desc" )
                             .length() == 1
                    && !$( e ).hasClass( "asc" ) && !$( e ).hasClass( "desc" ) )
                    || 1 < getDTable().order()
                                      .getSize( getDTable().isFrozen( column ) ) )
                {
                    Element item = DOM.createDiv();
                    item.setId( ContextMenuItems.ORDER_REMOVE_ALL.id() );
                    item.setInnerText( getDTable().translation()
                                                  .get( ContextMenuItems.ORDER_REMOVE_ALL.description() ) );
                    contextMenuElement.appendChild( item );
                }
            }

            // freeze/unfreeze column
            if ( getDTable().config()
                            .isFreezeUnfreezeColumnsEnabled()
                && getDTable().config()
                              .isFrozenTableEnabled() )
            {
                Element item = DOM.createDiv();
                item.setId( !getDTable().isFrozen( column ) ? ContextMenuItems.FREEZE.id()
                                : ContextMenuItems.UNFREEZE.id() );
                item.setInnerText( ( !getDTable().isFrozen( column ) ? dTable.translation()
                                                                             .get( ContextMenuItems.FREEZE.description() )
                                : getDTable().translation()
                                             .get( ContextMenuItems.UNFREEZE.description() ) ).replace( "_",
                                                                                                        cellValue ) );

                if ( !getDTable().isFrozen( column ) && getDTable().config()
                                                                   .isFreezeEnabled( column )
                    && 1 < getDTable().getColumnsVisible()
                                      .size() )
                {
                    item.setInnerText( getDTable().translation()
                                                  .get( ContextMenuItems.FREEZE.description() )
                                                  .replace( "_", cellValue ) );
                    contextMenuElement.appendChild( item );
                }
                else if ( getDTable().isFrozen( column ) && getDTable().config()
                                                                       .isUnfreezeEnabled( column )
                    && ( ( getDTable().config()
                                      .isEmptyFrozenTableEnabled()
                        && !getDTable().isAnyFrozenColumnHidden() && dTable.getColumnsFrozen()
                                                                           .size() >= 1 )
                        || ( !dTable.config()
                                    .isEmptyFrozenTableEnabled()
                            && dTable.getColumnsFrozen()
                                     .size() > 1 ) ) )
                {
                    item.setInnerText( getDTable().translation()
                                                  .get( ContextMenuItems.UNFREEZE.description() )
                                                  .replace( "_", cellValue ) );
                    contextMenuElement.appendChild( item );
                }
            }

            // hide column
            if ( getDTable().config()
                            .isHideUnhideColumnsEnabled()
                && getDTable().config()
                              .isColumnHideEnabled( column ) )
            {
                Element item = DOM.createDiv();
                item.setId( ContextMenuItems.HIDE.id() );
                item.setInnerText( getDTable().translation()
                                              .get( ContextMenuItems.HIDE.description() )
                                              .replace( "_", cellValue ) );
                if ( !getDTable().isFrozen( column ) && 1 < getDTable().getColumnsVisible()
                                                                       .size() )
                {
                    contextMenuElement.appendChild( item );
                }
                else if ( getDTable().isFrozen( column ) && 1 < getDTable().getColumnsFrozen()
                                                                           .size() )
                {
                    contextMenuElement.appendChild( item );
                }
            }
        }
        else if ( 3 <= id.length )
        // table body
        {
            column = Integer.parseInt( id[2] );
            row = Integer.parseInt( id[1] );

            if ( getDTable().filter()
                            .isFilterEnabled( column ) )
            {
                int menuDepth = getDTable().isFrozen( column ) ? getDTable().config()
                                                                            .getContextMenuDepthForFilteredFrozenTable()
                                : getDTable().config()
                                             .getContextMenuDepthForFilteredTable();
                if ( cellValue != null && !cellValue.isEmpty() && !"&nbsp;".equals( cellValue ) //
                    && !getDTable().filter()
                                   .filterOperatorExists( column, ContextMenuItems.IS_EMPTY.description() )
                    && !getDTable().search()
                                   .filterOperatorExists( column, ContextMenuItems.IS_EMPTY.description() ) )
                {
                    if ( getDTable().config()
                                    .isColumnFormatted( column )
                        && !getDTable().filter()
                                       .filterOperatorExists( column, ContextMenuItems.EQUAL.description() )
                        && !getDTable().search()
                                       .filterOperatorExists( column, ContextMenuItems.IS_EMPTY.description() ) )
                    {
                        menuItems.addAll( Arrays.asList( ContextMenuItems.EQUAL, ContextMenuItems.NOT_EQUAL,
                                                         ContextMenuItems.GREATER, ContextMenuItems.GREATER_EQUAL,
                                                         ContextMenuItems.LESS, ContextMenuItems.LESS_EQUAL ) );
                    }
                    else if ( !getDTable().config()
                                          .isColumnFormatted( column )
                        && !getDTable().filter()
                                       .filterOperatorExists( column, ContextMenuItems.LIKE.description() )
                        && !getDTable().search()
                                       .filterOperatorExists( column, ContextMenuItems.IS_EMPTY.description() ) )
                    {
                        menuItems.addAll( Arrays.asList( ContextMenuItems.LIKE, ContextMenuItems.LIKE_LEFT,
                                                         ContextMenuItems.LIKE_RIGHT, ContextMenuItems.LIKE_LEFT_RIGHT,
                                                         ContextMenuItems.NOT_LIKE, ContextMenuItems.NOT_LIKE_LEFT,
                                                         ContextMenuItems.NOT_LIKE_RIGHT,
                                                         ContextMenuItems.NOT_LIKE_LEFT_RIGHT ) );
                    }
                }

                if ( !getDTable().filter()
                                 .isFiltered( column ) )
                {
                    menuItems.addAll( Arrays.asList( ContextMenuItems.IS_EMPTY, ContextMenuItems.IS_NOT_EMPTY ) );
                }

                for ( ContextMenuItems menuItem : menuItems )
                {
                    String filter = getDTable().translation()
                                               .get( menuItem.description() );
                    filter = ( filter != null ? filter : menuItem.description() ).replace( "_", cellValue );

                    if ( menuItem.description()
                                 .equals( ContextMenuItems.IS_EMPTY.description() ) )
                    {
                        if ( !getDTable().filter()
                                         .filterExists( column, ContextMenuItems.IS_NOT_EMPTY.description() ) )
                        {
                            Element item = DOM.createDiv();
                            item.setId( menuItem.id() );
                            item.setInnerText( filter );
                            contextMenuElement.appendChild( item );
                        }
                    }
                    else if ( !menuItem.description()
                                       .equals( ContextMenuItems.LIKE_LEFT_RIGHT.description() )
                        && !menuItem.description()
                                    .equals( ContextMenuItems.NOT_LIKE_LEFT_RIGHT.description() ) )
                    {
                        if ( !getDTable().filter()
                                         .filterExists( column, filter ) )
                        {
                            Element item = DOM.createDiv();
                            item.setId( menuItem.id() );
                            item.setInnerText( filter );
                            contextMenuElement.appendChild( item );
                        }
                    }
                    else if ( menuItem.description()
                                      .equals( ContextMenuItems.LIKE_LEFT_RIGHT.description() ) )
                    {
                        for ( int i = 1; i <= menuDepth && i <= cellValue.length(); i++ )
                        {
                            filter = menuItem.description()
                                             .replace( "_", cellValue.substring( 0, i ) );
                            if ( !getDTable().filter()
                                             .filterExists( column, filter ) )
                            {
                                Element item = DOM.createDiv();
                                item.setId( menuItem.id() + i );
                                item.setInnerText( filter );
                                contextMenuElement.appendChild( item );
                            }
                        }
                    }
                    else if ( menuItem.description()
                                      .equals( ContextMenuItems.NOT_LIKE_LEFT_RIGHT.description() ) )
                    {
                        for ( int i = 1; i <= menuDepth && i <= cellValue.length(); i++ )
                        {
                            filter = menuItem.description()
                                             .replace( "_", cellValue.substring( 0, i ) );
                            if ( !getDTable().filter()
                                             .filterExists( column, filter ) )
                            {
                                Element item = DOM.createDiv();
                                item.setId( menuItem.id() + i );
                                item.setInnerText( filter );
                                contextMenuElement.appendChild( item );
                            }
                        }
                    }
                }
            }

            if ( getDTable().filter()
                            .isFiltered( column )
                || getDTable().search()
                              .isSearched( column ) )
            {
                String headerId =
                    "table#dt-" + ( getDTable().isFrozen( column ) ? "frozen-" : "" ) + "header-table" + id[0] + " tr";
                String header = $( headerId ).children( "td [id='" + id[0] + "h:0:" + id[2] + "']" )
                                             .children( "div" )
                                             .first()
                                             .text();
                Element item = DOM.createDiv();
                item.setId( ContextMenuItems.FILTER_REMOVE.id() );
                item.setInnerText( getDTable().translation()
                                              .get( ContextMenuItems.FILTER_REMOVE.description() )
                                              .replace( "_", header ) );
                contextMenuElement.appendChild( item );
            }
        }
    }

    @Override
    public void hide()
    {
        $( "#dt-menu-glass" ).remove();
        super.hide();
    }

    @Override
    protected void onPreviewNativeEvent( NativePreviewEvent event )
    {
        if ( event.getTypeInt() == Event.ONKEYDOWN && event.getNativeEvent()
                                                           .getKeyCode() == KeyCodes.KEY_ESCAPE )
        {
            hide();
        }
        super.onPreviewNativeEvent( event );
    }

    @Override
    public void onBrowserEvent( Event event )
    {
        event.stopPropagation();

        switch ( DOM.eventGetType( event ) )
        {
            case Event.ONCLICK:
                hide();
                String menuItem = DOM.eventGetTarget( event )
                                     .getId();
                if ( menuItem.startsWith( ContextMenuItems.LIKE_LEFT_RIGHT.id() ) )
                {
                    menuItem = ContextMenuItems.LIKE_LEFT_RIGHT.id();
                }
                else if ( menuItem.startsWith( ContextMenuItems.NOT_LIKE_LEFT_RIGHT.id() ) )
                {
                    menuItem = ContextMenuItems.NOT_LIKE_LEFT_RIGHT.id();
                }

                switch ( ContextMenuItems.item( menuItem ) )
                {
                    case ORDER_ASCENDING:
                        getDTable().order()
                                   .setOrder( $( targetE ), "asc" );
                        getDTable().sendMessage( "order", getDTable().order()
                                                                     .getOrders(),
                                                 true );
                        break;
                    case ORDER_DESCENDING:
                        getDTable().order()
                                   .setOrder( $( targetE ), "desc" );
                        getDTable().sendMessage( "order", getDTable().order()
                                                                     .getOrders(),
                                                 true );
                        break;
                    case ORDER_REMOVE:
                        getDTable().order()
                                   .setOrder( $( targetE ), "remove" );
                        getDTable().sendMessage( "order", getDTable().order()
                                                                     .getOrders(),
                                                 true );
                        break;
                    case ORDER_REMOVE_ALL:
                        getDTable().order()
                                   .setOrder( $( targetE ), null );
                        getDTable().sendMessage( "order", getDTable().order()
                                                                     .getOrders(),
                                                 true );
                        break;
                    case EQUAL:
                    case NOT_EQUAL:
                    case GREATER:
                    case GREATER_EQUAL:
                    case LESS:
                    case LESS_EQUAL:
                    case LIKE:
                    case NOT_LIKE:
                    case LIKE_LEFT:
                    case LIKE_RIGHT:
                    case LIKE_LEFT_RIGHT:
                    case NOT_LIKE_LEFT:
                    case NOT_LIKE_RIGHT:
                    case NOT_LIKE_LEFT_RIGHT:
                        cellValue = cellValue.isEmpty() ? " " : cellValue;
                        // column, row, operator, filter, cell value
                        getDTable().filter()
                                   .newFilter( column, row, ContextMenuItems.item( menuItem )
                                                                            .operator(),
                                               DOM.eventGetTarget( event )
                                                  .getInnerText(),
                                               cellValue );
                        break;
                    case IS_EMPTY:
                    case IS_NOT_EMPTY:
                        // column, row, operator, filter, cell value
                        getDTable().filter()
                                   .newFilter( column, row, ContextMenuItems.item( menuItem )
                                                                            .operator(),
                                               DOM.eventGetTarget( event )
                                                  .getInnerText(),
                                               " " );
                        break;
                    case FREEZE:
                    case UNFREEZE:
                        getDTable().setFrozen( column );
                        break;
                    case HIDE:
                        getDTable().sendMessage( "hideColumn", Integer.toString( column ), true );
                        break;
                    case FILTER_REMOVE:
                        getDTable().filter()
                                   .removeFilter( column );
                        break;
                    case NONE:
                    default:
                        break;
                }
                break;
            case Event.ONCONTEXTMENU:
                event.preventDefault();
                break;
            default:
                break;
        }
    }

    @Override
    public void show()
    {
        if ( 0 < contextMenuElement.getChildCount() )
        {
            super.show();

            int popupLeft = ( getAbsoluteLeft() + getOffsetWidth() + 5 ) <= Window.getClientWidth() ? getPopupLeft()
                            : ( Window.getClientWidth() - getOffsetWidth() - 5 );
            int popupTop = ( getAbsoluteTop() + getOffsetHeight() + 5 ) <= Window.getClientHeight() ? getPopupTop()
                            : ( Window.getClientHeight() - getOffsetHeight() - 5 );

            setPopupPosition( popupLeft, popupTop );
        }
    }

}
