/**
 * Copyright (C) 2012-2016  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.config.formatting;

import java.io.Serializable;
import java.text.DecimalFormat;
import java.text.Format;
import java.text.SimpleDateFormat;

import org.dussan.vaadin.dtable.config.ColumnType;

public class Formatting
    implements Serializable
{

    private static final long serialVersionUID = -3093581670172344937L;

    private Integer column = null;

    private Format format = null;

    private Formatting()
    {
        // Auto-generated constructor stub
    }

    public Formatting( int column, Format format )
    {
        this();
        this.column = column;
        this.format = format;
    }

    public Integer getColumn()
    {
        return column;
    }

    public void setColumn( Integer column )
    {
        this.column = column;
    }

    public static Format getFormat( ColumnType columnType )
    {
        switch ( columnType )
        {
            case BIG_DECIMAL:
            case DOUBLE:
            case FLOAT:
                return new DecimalFormat( "#,##0.0" );
            case BIG_INTEGER:
            case INTEGER:
            case LONG:
                return new DecimalFormat( "#,##0" );
            case DATE:
                return new SimpleDateFormat( "EEE MMM dd HH:mm:ss zzz yyyy" );
            case DECIMAL_NUMBER:
                return new DecimalFormat( "#0.0" );
            case INTEGER_NUMBER:
                return new DecimalFormat( "#0" );
            case TIMESTAMP:
                return new SimpleDateFormat( "yyyy.MM.dd.HH.mm.ss" );
            case BOOLEAN:
            case MEMO:
            case TEXT:
            default:
                return null;
        }
    }

    public Format getFormat()
    {
        return format;
    }

    public void setFormat( Format format )
    {
        this.format = format;
    }

    public String getPattern()
    {
        if ( format != null )
        {
            if ( format instanceof DecimalFormat )
            {
                return ( (DecimalFormat) format ).toPattern();
            }
            else if ( format instanceof SimpleDateFormat )
            {
                return ( (SimpleDateFormat) format ).toPattern();
            }
        }
        return null;
    }

}
