/**
 * Copyright (C) 2012-2016  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.processors;

import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.Format;
import java.text.MessageFormat;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.dussan.vaadin.dtable.DTable;
import org.dussan.vaadin.dtable.config.formatting.Formatting;

public class FormattingProcessor
{

    private FormattingProcessor()
    {
        // Auto-generated constructor stub
    }

    public static String[] process( Formatting formatting, List<Object> columnsData )
    {
        Format tempFormat = null;
        if ( formatting != null )
        {
            if ( formatting.getFormat() instanceof NumberFormat )
            {
                tempFormat = (NumberFormat) formatting.getFormat();
            }
            else if ( formatting.getFormat() instanceof DecimalFormat )
            {
                tempFormat = (DecimalFormat) formatting.getFormat();
            }
            else if ( formatting.getFormat() instanceof DateFormat )
            {
                tempFormat = (DateFormat) formatting.getFormat();
            }
            else if ( formatting.getFormat() instanceof MessageFormat )
            {
                tempFormat = (MessageFormat) formatting.getFormat();
            }
            else if ( formatting.getFormat() instanceof SimpleDateFormat )
            {
                tempFormat = (SimpleDateFormat) formatting.getFormat();
            }
        }

        List<String> tempData = new ArrayList<>();
        for ( int row = 0; row < columnsData.size(); row++ )
        {
            if ( columnsData.get( row ) == null )
            {
                tempData.add( "" );
            }
            else if ( tempFormat != null )
            {
                try
                {
                    if ( tempFormat instanceof MessageFormat )
                    {
                        tempData.add( ( "" + columnsData.get( row ) ).trim() );
                    }
                    else
                    {
                        tempData.add( tempFormat.format( columnsData.get( row ) ) );
                    }
                }
                catch ( Exception e )
                {
                    Logger.getLogger( DTable.class.getName() )
                          .log( Level.INFO, e.getLocalizedMessage(), e );
                    tempData.add( ( "" + columnsData.get( row ) ).trim() );
                }
            }
            else
            {
                tempData.add( ( "" + columnsData.get( row ) ).trim() );
            }
        }
        return tempData.toArray( new String[] {} );
    }

    public static String process( Formatting formatting, Object data )
    {
        if ( formatting != null )
        {
            try
            {
                if ( formatting.getFormat() instanceof NumberFormat )
                {
                    return ( (NumberFormat) formatting.getFormat() ).format( data );
                }
                else if ( formatting.getFormat() instanceof DecimalFormat )
                {
                    return ( (DecimalFormat) formatting.getFormat() ).format( data );
                }
                else if ( formatting.getFormat() instanceof DateFormat )
                {
                    return ( (DateFormat) formatting.getFormat() ).format( data );
                }
                else if ( formatting.getFormat() instanceof SimpleDateFormat )
                {
                    return ( (SimpleDateFormat) formatting.getFormat() ).format( data );
                }
            }
            catch ( Exception e )
            {
                Logger.getLogger( DTable.class.getName() )
                      .log( Level.INFO, e.getLocalizedMessage(), e );
            }
        }
        return data != null ? data.toString() : "";
    }

    public static Object partialProcess( Formatting formatting, String data )
    {
        if ( formatting != null )
        {
            try
            {
                if ( formatting.getFormat() instanceof NumberFormat )
                {
                    return NumberFormat.getInstance()
                                       .parse( data );
                }
                else if ( formatting.getFormat() instanceof DecimalFormat )
                {
                    return DecimalFormat.getInstance()
                                        .parse( data );
                }
                else if ( formatting.getFormat() instanceof DateFormat )
                {
                    return DateFormat.getInstance()
                                     .parse( data );
                }
                else if ( formatting.getFormat() instanceof SimpleDateFormat )
                {
                    return SimpleDateFormat.getInstance()
                                           .parse( data );
                }
            }
            catch ( Exception e )
            {
                Logger.getLogger( DTable.class.getName() )
                      .log( Level.INFO, e.getLocalizedMessage(), e );
            }
        }
        return data;
    }

    public static Object process( Formatting formatting, String data )
    {
        if ( formatting != null )
        {
            try
            {
                if ( formatting.getFormat() instanceof NumberFormat )
                {
                    return ( (NumberFormat) formatting.getFormat() ).parse( data );
                }
                else if ( formatting.getFormat() instanceof DecimalFormat )
                {
                    return ( (DecimalFormat) formatting.getFormat() ).parse( data );
                }
                else if ( formatting.getFormat() instanceof DateFormat )
                {
                    return ( (DateFormat) formatting.getFormat() ).parse( data );
                }
                else if ( formatting.getFormat() instanceof SimpleDateFormat )
                {
                    return ( (SimpleDateFormat) formatting.getFormat() ).parse( data );
                }
            }
            catch ( Exception e )
            {
                Logger.getLogger( DTable.class.getName() )
                      .log( Level.INFO, e.getLocalizedMessage(), e );
            }
        }
        return data;
    }

}
